# Django Signals Ahoy
# Copyright Bruce Kroeze 2009
#
# Released under the "New BSD License"
# Please see LICENSE.rst in this distribution for license details.
# -----------------
"""
Useful signals:

 - `application_search`: sent by search views to ask all listeners to add search results::

    Ex: application_search.send(Sender, request=request, keywords=keywords, results={})

 - `collect_urls`: sent by urls modules to allow listeners to add urls to that module::

    Ex: collect_urls.send(sender=MODULE, patterns=urlpatterns)

- `form_initialdata`: sent by forms when collecting initial data::

    Ex: form_initialdata.send(sender=FORM, initial={}, [arg=val, arg2=val2, ...])

 - `form_init`: sent by forms when adding fields::

    Ex: form_init.send(sender=FORM, [arg=val, arg2=val2, ...])

 - `form_presave`: sent by forms just prior to saving::

    Ex: form_presave.send(sender=FORM, [arg=val, arg2=val2, ...])

 - `form_postsave`: sent by forms just after saving::

    Ex: form_postsave.send(sender=FORM, [arg=val, arg2=val2, ...])
    
- `form_validate`: sent by forms prior to validation::

    Ex: form_validate.send(sender=FORM, form=FORM)

 - `view_prerender`: sent by views just before rendering::

    Ex: view_prerender.send(sender=VIEW, context=CONTEXT)

"""
import django.dispatch
application_search = django.dispatch.Signal()
collect_urls = django.dispatch.Signal()
form_init = django.dispatch.Signal()
form_initialdata = django.dispatch.Signal()
form_presave = django.dispatch.Signal()
form_postsave = django.dispatch.Signal()
form_validate = django.dispatch.Signal()
view_prerender = django.dispatch.Signal()



# Copyright (C) 2007-2009 Etienne Robillard <robillard.etienne@gmail.com>
# All rights reserved.
#
# Please see the "LICENSE" and "NOTICE" files included in the source 
# distribution for details on licensing info.
"""Schema for MovieReviews."""

from schevo.schema import *
schevo.schema.prep(locals())


class SchevoIcon(E.Entity):

    _hidden = True

    name = f.string()
    data = f.image()

    _key(name)


class Actor(E.Entity):

    name = f.string()

    _key(name)

    def x_movies(self):
        return [casting.movie
                for casting in self.m.movie_castings()]


class Director(E.Entity):

    name = f.string()

    _key(name)


class Movie(E.Entity):

    title = f.string()
    release_date = f.date()
    director = f.entity('Director')
    description = f.string(multiline=True, required=False)

    _key(title)

    def x_actors(self):
        return [casting.actor
                for casting in self.m.movie_castings()]

    def __unicode__(self):
        return u'%s (%i)' % (self.title, self.release_date.year)


class MovieCasting(E.Entity):

    movie = f.entity('Movie', CASCADE)
    actor = f.entity('Actor')

    _key(movie, actor)


E.Actor._sample = [
    ('Keanu Reeves', ),
    ('Winona Ryder', ),
    ]

E.Director._sample = [
    ('Richard Linklater', ),
    ('Stephen Herek', ),
    ('Tim Burton', ),
    ]

E.Movie._sample = [
    ('A Scanner Darkly', '2006-07-28', ('Richard Linklater', ),
        DEFAULT),
    ("Bill & Ted's Excellent Adventure", '1989-02-17',
        ('Stephen Herek', ), DEFAULT),
    ('Edward Scissorhands', '1990-12-14', ('Tim Burton', ),
        DEFAULT),
    ]

E.MovieCasting._sample = [
    (('A Scanner Darkly', ), ('Keanu Reeves', )),
    (('A Scanner Darkly', ), ('Winona Ryder', )),
    (("Bill & Ted's Excellent Adventure", ), ('Keanu Reeves', )),
    (('Edward Scissorhands', ), ('Winona Ryder', )),
    ]



"""Misc template backends and utilities"""
import importlib
from notmm.utils.django_settings import LazySettings

_settings = LazySettings()

__all__ = [
    'available_backends',
    'TemplateLoaderFactory',] 

class TemplateLoaderFactory(object):
    """A proxy class which delegates access to a template loader instance"""

    loader_instance = None

    def __init__(self, label=''):
        self._label = label # Backend name (optional)

    def __call__(self, **kwargs):
        return getattr(self, 'loader_instance').__call__(**kwargs)

    def __repr__(self):
        return "<TemplateLoaderFactory: %s>" % self.get_loader()

    @classmethod
    def get_loader(cls):
        """Returns the currently defined template
        loader backend instance."""

        return getattr(cls, 'loader_instance')
    
    @classmethod
    def set_loader(cls, attr, value):
        """Set the class ``loader_instance`` attribute, 
        or raise an exception if loader is invalid/unusable."""
        setattr(cls, attr, value)
        return None

    @classmethod
    def configure(cls, 
    #backend='mako',
    template_loader_class=_settings.TEMPLATE_LOADER,
    kwargs={}):
        """
        Setup a default template_loader instance for template loading and 
        rendering. Such an inheritable object should have a ``get_template`` 
        method for properly looking up templates. 
        """
        # 'mako.CachedTemplateLoader' -> notmm.utils.template.backend.i
        try:
            mod, name = template_loader_class.rsplit('.', 1)
            instance = getattr(importlib.import_module(mod), \
                name).__call__(**kwargs)
        except Exception:
            raise
        else:
            cls.set_loader('loader_instance', instance)
        return None

available_backends = {
    'mako': TemplateLoaderFactory(label='Mako 0.8.1')
}

if __name__ == '__main__':
    print("available backends:")
    for backend in available_backends:
        print(backend)



# Copyright (C) 2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from gazpacho import gapi
from gazpacho.cursor import Cursor
from gazpacho.gadget import Gadget
from gazpacho.widgets.base.base import ContainerAdaptor

class FixedAdaptor(ContainerAdaptor):
    def __init__(self):
        # save the last button click coordinate
        self.x = 0
        self.y = 0

    def post_create(self, context, fixed, interactive=True):
        fixed.connect('expose-event', self.on_expose_event)

    def replace_child(self, context, current, new, container):
        "'current' is always None for Fixed since it never has any placeholder"
        container.put(new, self.x, self.y)

    def button_release(self, context, fixed, event):
        # we remember the point where the user clicked
        toplevel = fixed.get_toplevel()
        if toplevel:
            self.x, self.y = toplevel.translate_coordinates(fixed,
                                                            int(event.x),
                                                            int(event.y))

        # if there is some widget selected in the palette it's time to add it
        project = context.get_project()
        if project._app.add_class:
            gadget = Gadget.from_widget(fixed)
            gapi.create_gadget(project, project.get_app().add_class,
                               None, gadget)
        return False

    def motion_notify(self, context, fixed, event):
        app = context.get_project()._app
        Cursor.set_for_widget_adaptor(event.window, app.add_class)
        return False

    def on_expose_event(self, fixed, event):
        # draw a grid
        window = fixed.window
        bw = fixed.get_property('border-width')
        w, h = window.get_size()
        gc = fixed.style.dark_gc[gtk.STATE_NORMAL]
        for gridx in range(bw, w-bw, 10):
            for gridy in range(bw, h-bw, 10):
                window.draw_point(gc, gridx, gridy)

        return False




from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from library import get_library
import re

ENABLED = 0
DISABLED = 1
REMOVED = 2

STATUS_CHOICES = (
    (ENABLED,  _('Enabled')),
    (DISABLED, _('Disabled')),
    (REMOVED,  _('Removed'))
)

LABEL_RE = re.compile('^[a-zA-Z_][a-zA-Z_0-9.]*$')

def is_valid_label(name):
    return bool(LABEL_RE.match(name))

def construct_template_path(app, name, ext='.html'):
    if not is_valid_label(name): raise RuntimeError, u"invalid label: " + name
    if not is_valid_label(app): raise RuntimeError, u"invalid label: " + app
    return '/'.join([app.split('.')[-1], 'plugins', name.replace('.','/')])+ext


class PluginPoint(models.Model):
    label      = models.CharField(max_length=255, unique=True,
                    help_text=_("The label for the plugin point."))
    index      = models.IntegerField(default=0)

    registered = models.BooleanField(default=False,
                    help_text=_("is this a registered plugin point with a "
                                "library entry?"))
    status     = models.SmallIntegerField(choices=STATUS_CHOICES,
                                          default=ENABLED)

    class Meta:
        ordering = ('index', 'id')

    def __unicode__(self):
        return u'plugin_point:' + self.label

    @property
    def app(self):
        if not self.registered: return ''
        return self.label.rsplit('.', 1)[0]

    @property
    def name(self):
        return self.label.rsplit('.', 1)[-1]

    def get_plugins(self, user=None):
        """get all the plugins in appropriate order"""
        if self.status: return []
        if user is None:
            return self.plugin_set.filter(status=ENABLED).order_by('index','id')
        upref = user.userpluginpreference_set.filter(plugin__status=ENABLED,
                plugin__point=self) #.order_by('index', 'id').select_related()
        visible = [up.plugin for up in
                       upref.filter(Q(visible=True)|Q(plugin__required=True))]
        plugins = self.plugin_set.filter(status=ENABLED).exclude(
            id__in=[x['plugin'] for x in upref.values('plugin')]
            ).order_by('index', 'id')
        return visible + list(plugins)

    def get_options(self):
        if not self.registered: return {}
        lib = get_library(self.app)
        call = lib.get_plugin_point_call(self.name)
        return call.options

    # __call__ not allowed
    def call(self, context, user, **args):
        if not self.registered: return {}
        lib = get_library(self.app)
        call = lib.get_plugin_point_call(self.name)
        options = call.options
        base = [self,]
        if options.get('takes_context', False):
            base.append(context)
        if options.get('takes_user', False):
            base.append(user)
        if options.get('takes_args', False):
            return call(*base, **args)
        return call(self, *base)

class Plugin(models.Model):
    point      = models.ForeignKey(PluginPoint)
    label      = models.CharField(max_length=255, unique=True,
                    help_text=_("The label for the plugin point."))

    index      = models.IntegerField(default=0)

    registered = models.BooleanField(default=False,
                    help_text=_("is this a registered plugin?"))
    status     = models.SmallIntegerField(choices=STATUS_CHOICES,
                                          default=ENABLED)

    required   = models.BooleanField(default=False,
                    help_text=_("users can not remove this plugin."))
    template   = models.TextField(
                    help_text=_("template to load for the plugin."))

    class Meta:
        order_with_respect_to = 'point'
        ordering = ('point', 'index', 'id')

    def __unicode__(self):
        return u'plugin:' + self.label

    @property
    def app(self):
        #if not self.registered: return ''
        return self.label[:-(len(self.point.label)+1)]

    @property
    def name(self):
        return self.point.label

    def get_options(self):
        if not self.registered: return {}
        lib = get_library(self.app)
        call = lib.get_plugin_call(self.name)
        return call.options

    #__call__ not allowed...
    def call(self, context, user, **args):
        if not self.registered: return {}
        lib = get_library(self.app)
        call = lib.get_plugin_call(self.name)
        options = call.options
        base = [self,]
        if options.get('takes_context', False):
            base.append(context)
        if options.get('takes_user', False):
            base.append(user)
        if options.get('takes_args', False):
            return call(*base, **args)
        return call(self, *base)

class UserPluginPreference(models.Model):
    user    = models.ForeignKey(User)
    plugin  = models.ForeignKey(Plugin)
    visible = models.BooleanField(default=True)
    index   = models.IntegerField(default=0)

    class Meta:
        unique_together = ['user', 'plugin']
        order_with_respect_to = 'plugin'
        ordering = ('plugin', 'user', 'index', 'id')

    def __unicode__(self):
        return u':'.join(['pluginpref', self.user.username, self.plugin.label])



import shipper
from livesettings import config_choice_values

def get_methods():
    return [shipper.Shipper(service_type=value) for value in config_choice_values('shipping.modules.usps', 'USPS_SHIPPING_CHOICES')]



"""
URLConf for Caching app
"""

from django.conf.urls.defaults import *

urlpatterns = patterns('keyedcache.views',
    (r'^$', 'stats_page', {}, 'keyedcache_stats'),
    (r'^view/$', 'view_page', {}, 'keyedcache_view'),
    (r'^delete/$', 'delete_page', {}, 'keyedcache_delete'),
)



"""
This module requires that the tcclink library and the associated python bindings are installed.
You can get it from your TrustCommerce account d/l links or from -
http://www.trustcommerce.com/tclink.html
"""

from django.utils.translation import ugettext_lazy as _
from payment.modules.base import BasePaymentProcessor, ProcessorResult, NOTSET
import tclink

class PaymentProcessor(BasePaymentProcessor):
    # TrustCommerce payment processing module
    # You must have an account (or a test/trial account) in order to use this module
    # note that although this can be done using a url post like AuthorizeNet, 
    # the tclink method shown here is extremely simple, and more secure, and uses
    # backp servers guaranteeing uptime versus the singe-server https method -  
    # see TC for details.
    
    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('trustcommerce', settings)
        self.demo = 'y'
        self.AVS = 'n'
        self.custid = settings.LOGIN.value
        self.password = settings.PASSWORD.value
        if settings.LIVE.value:
            self.demo = 'n'
        if settings.AVS.value:
            self.AVS = 'y'
        self.auth = settings.AUTH_TYPE.value
        self.tclink_version = tclink.getVersion()

    def prepare_post(self, data, amount):
        # See tclink developer's guide for additional fields and info
        # convert amount to cents, no decimal point
        amount = unicode((amount * 100).to_integral()) 

        # convert exp date to mmyy from mm/yy or mm/yyyy
        cc = data.credit_card 
        exp = u"%.2d%.2d" % (int(cc.expire_month), (int(cc.expire_year) % 100)) 

        invoice = "%s" % data.id
        failct = data.paymentfailures.count()
        if failct > 0:
            invoice = "%s_%i" % (invoice, failct)

        self.transactionData = {
            # account data
            'custid'	: self.custid,
            'password'	: self.password,
            'demo'	: self.demo,

            # Customer data
            'name'  	: data.contact.first_name + u' ' + data.contact.last_name,
            'address1'	: data.full_bill_street,
            'city'	: data.bill_city,
            'state' 	: data.bill_state,
            'zip' 	:data.bill_postal_code,
            'country'	: data.bill_country,
            'phone' 	: data.contact.primary_phone.phone,
            # other possibiliities include email, ip, offlineauthcode, etc 

            # transaction data
            'media'     : 'cc',
            'action'	: self.auth,  	# or 'sale', 'credit', etc - see tclink dev guide
            'amount' 	: amount,	# in cents
            'cc'	: cc.decryptedCC,  # use '4111111111111111' for test
            'exp'	: exp, 		# 4 digits eg 0108
            'cvv'	: cc.ccv,
            'avs'	: self.AVS,		# address verification - see tclink dev guide
            'ticket'	: u'Order: %s ' % invoice,
            'operator'	: 'Satchmo'
            }
        for key, value in self.transactionData.items(): 
            if isinstance(value, unicode): 
                self.transactionData[key] = value.encode('utf7',"ignore") 
        
    def capture_payment(self, testing=False, order=None, amount=NOTSET):
        """process the transaction through tclink"""
        if not order:
            order = self.order

        if amount == NOTSET:
            amount = order.balance
            
        if order.paid_in_full:
            self.log_extra('%s is paid in full, no capture attempted.', order)
            self.record_payment()
            return ProcessorResult(self.key, True, _("No charge needed, paid in full."))

        self.log_extra('Capturing payment for %s', order)

        self.prepare_post(order, amount)

        result = tclink.send(self.transactionData)
        status = result ['status']
        payment = None
        success = False
        
        if status == 'approved':
            payment = self.record_payment(order=order, amount=amount, 
                transaction_id="", reason_code=status)
            success = True
            msg = unicode(result)

        else:
            if status == 'decline':
                msg = _(u'Transaction was declined.  Reason: %s' % result['declinetype'])
                failmsg = u'Transaction was declined.  Reason: %s' % result['declinetype']

            elif status == 'baddata':
                msg = _(u'Improperly formatted data. Offending fields: %s' % result['offenders'])
                failmsg = u'Improperly formatted data. Offending fields: %s' % result['offenders']

            else:
                status = "error"
                msg = _(u'An error occurred: %s' % result['errortype'])
                failmsg = u'An error occurred: %s' % result['errortype']

            payment = self.record_failure(order=order, amount=amount, 
                transaction_id="", reason_code=status, 
                details=failmsg)

        return ProcessorResult(self.key, success, msg, payment=payment)
        
if __name__ == "__main__":
    #####
    # This is for testing - enabling you to run from the command line & make sure everything is ok
    #####
    import os
    from livesettings import config_get_group 
    from decimal import Decimal
    
    # Set up some dummy classes to mimic classes being passed through Satchmo
    class testContact(object):
        pass
    class testCC(object):
        pass
    class phone(object):
        pass
    class testOrder(object):
        def __init__(self):
            self.contact = testContact()
            self.credit_card = testCC()
            self.contact.primary_phone = phone()
        def order_sucess(self):
            pass


    if not os.environ.has_key("DJANGO_SETTINGS_MODULE"):
        os.environ["DJANGO_SETTINGS_MODULE"]="satchmo_store.settings"
   
    sampleOrder = testOrder()
    sampleOrder.contact.first_name = 'Chris'
    sampleOrder.contact.last_name = 'Smith'
    sampleOrder.contact.primary_phone.phone = '801-555-9242'
    sampleOrder.full_bill_street = '123 Main Street'
    sampleOrder.bill_postal_code = '12345'
    sampleOrder.bill_state = 'TN'
    sampleOrder.bill_city = 'Some City'
    sampleOrder.bill_country = 'US'
    sampleOrder.balance = Decimal("27.00")
    sampleOrder.total = Decimal("27.00")
    sampleOrder.credit_card.decryptedCC = '4111111111111111'
    sampleOrder.credit_card.expire_month = "10"
    sampleOrder.credit_card.expire_year = "2010"
    sampleOrder.credit_card.ccv = "123"
    sampleOrder.credit_card.order = "987654"
    sampleOrder.credit_card.orderpayment_id = "123"
    
    trustcommerce_settings = config_get_group('PAYMENT_TRUSTCOMMERCE')

    processor = PaymentProcessor (trustcommerce_settings)
    processor.prepare_data(sampleOrder)
    results = processor.process()
    print results




from notmm.utils.django_settings import LazySettings
from notmm.utils.django_compat import get_callable
from importlib import import_module
class ContextPopException(Exception):
    "pop() has been called more times than push()"
    pass
class RequestContext(object):
    def __init__(self, request, ctx):

        self.settings = LazySettings()
        self.context_processors = self.settings.TEMPLATE_CONTEXT_PROCESSORS
        self.view_dicts = [ctx,]
        for name in self.context_processors:
            f = get_callable(name)
            result = f(request)
            self.view_dicts.append(result)

    def __getitem__(self, key):
        for d in reversed(self.view_dicts):
            if key in d:
                return d[key]
        raise KeyError(key)

    def update(self, dct):
        self.view_dicts.append(dct)
        return None
        
    def pop(self):
        if len(self.view_dicts) == 1:
            raise ContextPopException
        return self.view_dicts.pop()

   




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2009-2010 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
# 
"""Common utilities for working with Django settings modules."""

import os, sys, threading

from functools import wraps
from notmm.datastore import DataStore
from notmm.utils.django_compat import memoize


__all__ = ['SettingsProxy', 'LazySettings']


class SettingsProxy(object):
    """
    A settings delegator object which returns a thread-local instance
    used for holding Django settings.
    """

    _settings_cache = {}
    _settings_module_name = 'DJANGO_SETTINGS_MODULE'
    
    def __init__(self, modname=None, autoload=True):
        """Initialize default options for the SettingsProxy instance
        
        Optional attributes:

        ``autoload`` -- Set to ``False`` to disable settings autoloading. By
        default, allow settings to be autoloaded during the controller
        initialization time.
        """

        # ensure to initialize _settings_cache
        self._settings_cache.clear()
        self.initsettings(name=modname)
        #if modname in sys.modules:
        #    #print 'doing a coup..'
        #    self._settings_cache[modname] = sys.modules[modname]

        # set autoload flag for django settings
        self.autoload = autoload
               
    def get_settings(self):
        """ Returns a ready-to-use LocalStore instance. """
        try:
            _settings = self._settings_cache[self.modname]
        except KeyError:
            #print "module %s not found in cache!" % self.modname
            # Use the provided settings module name
            _settings = DataStore(self.modname, autoload=self.autoload)
    	# Install the settings object in _settings_cache
        else:
            if not self.modname in self._settings_cache:
                self._settings_cache[self.modname] = _settings
        return _settings   


    def __str__(self):
        return "<SettingsProxy: %s>" % str(self.settings)

    def __call__(self):
        
        lock = threading.Lock()

        with lock:
            lock.acquire()
            try:
                return self.get_settings()
            finally:
                lock.release()

    def destroysettings(self, name):
        self._settings_cache.clear()
        return None
    
    def initsettings(self, name=None):
        self.destroysettings(name)
        if name is None:
            self.modname = os.environ[getattr(self, '_settings_module_name')]
        else:
            self.modname = name


    settings = property(memoize(get_settings, _settings_cache, 1))
    

class LazySettings(SettingsProxy):
    """Generic setting container based on SettingsProxy"""

    def __init__(self, **kwargs):
        if not 'autoload' in kwargs:
            kwargs['autoload'] = True
        super(LazySettings, self).__init__(**kwargs)

    def __new__(cls, *args, **kwargs):
        pyobj = object.__new__(cls, *args, **kwargs)
        pyobj.__init__()
        return pyobj.get_settings()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

"""Decorators functions returning a callable"""

from functools import wraps
from session import ScopedSession

__all__ = ('with_session',)

def with_session(engine=None):
    """
    Decorator function for attaching a `` Session`` instance
    as a keyword argument in ``request.environ``. 
    """
    def decorator(view_func):
        def _wrapper(request, *args, **kwargs):
            if engine is not None:
                #print "setting session"
                Session = ScopedSession(engine=engine)
                #Session.set_session(engine)
                request.environ['_scoped_session'] = Session.get_session()
            return view_func(request, *args, **kwargs)
        return wraps(view_func)(_wrapper)
    return decorator




from django.conf.urls.defaults import *
from livesettings import config_get_group

config = config_get_group('PAYMENT_PURCHASEORDER')

urlpatterns = patterns('payment',
    (r'^$', 'modules.purchaseorder.views.pay_ship_info', 
       {'SSL':config.SSL.value}, 'PURCHASEORDER_satchmo_checkout-step2'),

    (r'^confirm/$', 'modules.purchaseorder.views.confirm_info', 
       {'SSL':config.SSL.value}, 'PURCHASEORDER_satchmo_checkout-step3'),

     (r'^success/$', 'views.checkout.success', 
        {'SSL':config.SSL.value}, 'PURCHASEORDER_satchmo_checkout-success'),
)



#!/usr/bin/env python
# Copyright (C) 2010 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved
#
# <LICENSE>

import sys, os
import socket
import argparse
import datetime
import schevo.mt
import schevo.constant as constant
import schevo.entity   as entity
import schevo.fieldspec as fieldspec
import demjson
#import unicodedata
from notmm.dbapi.orm import schevo_compat
from notmm.utils.django_settings import LazySettings
settings = LazySettings()

from test_support import unittest, inittestpackage
inittestpackage()

from model import db


schevo.mt.install(db)

#db = schevo_compat.XdserverProxy(settings.DATABASE_NAME)
UNASSIGNED = constant.UNASSIGNED
escape_unicode = False

def extent_to_dict(instance, f="_field_spec"):
    """
    Convert a ``schevo.extent.Extent`` instance attributes to a flattened ``dict``
    object.

    """
    rv = {}
    for name, field in getattr(instance, f).items():
        #XXX: save the field type here? 
        obj = getattr(instance, name)
        if obj is UNASSIGNED: 
            # convert UNASSIGNED fields to None
            fdata = None
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            # convert datetime objects to a string type
            fdata = obj.strftime("%Y-%M-%d@%H:%S") # format?
        else:
            # follow foreing relationships
            if hasattr(obj, 's'):
                #field_spec = getattr(obj, f)
                fdata = obj.s.field_map()
                for k,v in fdata.items():
                    #fdata[k] = "%s"%unicodedata.normalize('NFC', unicode(v))
                    if not v is UNASSIGNED:
                        fdata[k] = str(type(v))
            else:
                fdata = obj
        rv[name] = fdata        
    return rv   

#TODO: relocate this as a bound method to RelationProxy (==0.4.3)
def dump_all(db):
    result = []
    info_map = {}
    for x in db.extents():
        #print x
        try:
            info_map[x.name] = [extent_to_dict(item) for item in x.find()]
        except AttributeError:
            raise
        result.append(info_map)
    return result

class FieldSpecTestCase(unittest.TestCase):

    def setUp(self):
        # New-style django settings loading
        #db._sync()
        #db.commit()
        pass

    def runTest(self):
        pass

    def test_dump_all(self):

        # Verify the database name
        info_map = dump_all(db)
        self.failUnlessEqual(isinstance(info_map, list), True)
        
        # save the info_map to a file in json format
        # json = demjson.encode(info_map, strict=True)
        #f = 'test.json'
        #open(f, 'w').write(json.decode())
        #os.unlink('test.json')




from django import template
from django.template.loader import render_to_string
from livesettings import config_value
from satchmo_ext.productratings.utils import get_product_rating_string, get_product_rating
import logging

log = logging.getLogger('shop.templatetags')

register = template.Library()

def product_rating_form(request, product, form):
    """Output our product comment form with the proper redirect in it."""
    log.debug('returning product rating form for %s', product)
    return {
        "form" : form,
        "product" : product,
        "user" : request.user,
    }

register.inclusion_tag('productratings/product_rating_form.html')(product_rating_form)

def product_ratings(context):
    """
    Display the ratings for a specific product.
    """
    shop = context['shop']
    rendered = render_to_string('productratings/_product_ratings.html', context)
    return { 'rendered_product_ratings': rendered }

register.inclusion_tag('productratings/_render_product_ratings.html', takes_context=True)(product_ratings)
        
def product_rating_average_string(product): 
    """Get the average product rating as a string, for use in templates"""   
    return get_product_rating_string(product)

register.filter("product_rating_average_string", product_rating_average_string)

def product_rating_average(product):  
    """Get the average product rating as a number"""  
    return get_product_rating(product)

register.filter("product_rating_average", product_rating_average)



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

# External API, used by extensions. Do not remove.
def get_bool_from_string_with_default(value, default):
    if value in ['True', 'TRUE', 'true', 'yes', '1']:
        return True
    elif value in ['False', 'FALSE', 'false', 'no', '0']:
        return False
    else:
        return default

def get_parent(widget):
    from gazpacho.gadget import Gadget
    parent_widget = widget
    gadget = None
    while True:
        parent_widget = parent_widget.get_parent()
        if parent_widget is None:
            return None
        gadget = Gadget.from_widget(parent_widget)
        if gadget is not None:
            return gadget

    return None

def select_iter(treeview, item_iter):
    """
    @param treeview:
    @param item_iter:
    """
    model = treeview.get_model()
    path = model[item_iter].path
    if treeview.flags() & gtk.REALIZED:
        treeview.expand_to_path(path)
        treeview.scroll_to_cell(path)
    treeview.get_selection().select_path(path)

def get_all_children(container):
    """This function get all children of a container, included its internal
    children"""
    children = []
    def fetch_child(child, bag):
        bag.append(child)
    container.forall(fetch_child, children)
    return children



from decimal import Decimal
from django import template
from django.utils.safestring import mark_safe
from l10n.utils import moneyfmt
from satchmo_utils.templatetags import get_filter_args
from shipping.modules.tiered.models import Carrier

register = template.Library()

def tiered_shipping(price, args=''):
    if not args:
        raise template.TemplateSyntaxError('tiered_shipping needs the name of the carrier, as value|tiered_shipping:"carrier"')
    
    try:
        c = Carrier.objects.get(key=args)
    except Carrier.DoesNotExist:
        raise template.TemplateSyntaxError('tiered_shipping needs the name of a valid carrier, could not find carrier "%s"' % args)
    shipping = c.price(Decimal(price))
    
    return mark_safe(moneyfmt(shipping))

register.filter(tiered_shipping)



#!/usr/bin/env python
# Copyright (C) 2013 Etienne Robillard
# All rights reserved.
# <LICENSE=APACHEV2>
"""Generic HTML form data extraction utilities"""

try:
    # Python 2.6 and greater
    from urlparse import parse_qsl
except ImportError:
    from cgi import parse_qsl

from operator import itemgetter
#import threading

__all__ = ['FormWrapper', 'MultiDict']

class FormWrapper(dict):
    """A low-level extension to copy HTML form 
    data into a ``dict`` object."""
    
    def __init__(self, data={}):
        # FieldStorage assumes a default CONTENT_LENGTH of -1
        try:
            self.content_length = int(data.get('CONTENT_LENGTH', -1))    
        except ValueError:
            self.content_length = -1
        self.environ = data.copy()
    

class MultiDict(FormWrapper):

    def _get_raw_post_data(self, input_key='wsgi.input'):
        rfile = self.environ[input_key]
        # p = rfile.read(self.content_length)
        #    for k,v in parse_qsl(p, keep_blank_values=1):
        #        params[k.strip()] = str(v)
        if rfile is None:
            return dict()
        data = parse_qsl(rfile.read(
            int(self.content_length)
            ))
        return dict(map(itemgetter(0,1), data))

    raw_post_data = property(_get_raw_post_data)




from django.template import Library
from django.conf import settings
from satchmo_store.contact.models import Contact
from livesettings import config_value
import sys

register = Library()

def show_tracker(secure=False):
    """
    Output the google tracker code.
    """
    return({"GOOGLE_CODE": config_value('GOOGLE', 'ANALYTICS_CODE'),
            "secure" : secure})

register.inclusion_tag("shop/google-analytics/tracker.html")(show_tracker)

def show_receipt(context):
    """
    Output our receipt in the format that Google Analytics needs.
    """
    return({'Store': settings.SITE_NAME,
            'order': context['order']})

register.inclusion_tag("shop/google-analytics/receipt.html", takes_context=True)(show_receipt)

def google_track_signup(context):
    """
    Output a a new user signup in the format that Google Analytics needs.
    """
    request = context['request']
    try:
        contact = Contact.objects.from_request(request, create=False)
    except Contact.DoesNotExist:
        contact = None

    return({'contact' : contact})

register.inclusion_tag("shop/google-analytics/signup.html", takes_context=True)(google_track_signup)

def google_adwords_sale(context):
    """
    Output our receipt in the format that Google Adwords needs.
    """
    order = context['order']
    try:
        request = context['request']
    except KeyError:
        print >> sys.stderr, "Template satchmo.show.templatetags.google.google_adwords_sale couldn't get the request from the context.  Are you missing the request context_processor?"
        return ""

    secure = request.is_secure()
    try:
        language_code = request.LANGUAGE_CODE
    except KeyError:
        language_code = settings.LANGUAGE_CODE

    return({"GOOGLE_ADWORDS_ID": config_value('GOOGLE', 'ADWORDS_ID'),
            'Store': settings.SITE_NAME,
            'value': order.total,
            'label': 'purchase',
            'secure' : secure,
            'language_code' : language_code})

register.inclusion_tag("shop/google-analytics/adwords_conversion.html", takes_context=True)(google_adwords_sale)

def google_adwords_signup(context):
    """
    Output signup info in the format that Google adwords needs.
    """
    request = context['request']
    try:
        request = context['request']
    except KeyError:
        print >> sys.stderr, "Template satchmo.show.templatetags.google.google_adwords_sale couldn't get the request from the context.  Are you missing the request context_processor?"
        return ""

    secure = request.is_secure()
    try:
        language_code = request.LANGUAGE_CODE
    except AttributeError:
        language_code = settings.LANGUAGE_CODE

    return({"GOOGLE_ADWORDS_ID": config_value('GOOGLE', 'ADWORDS_ID'),
            'Store': settings.SITE_NAME,
            'value': 1,
            'label': 'signup',
            'secure' : secure,
            'language_code' : language_code})

register.inclusion_tag("shop/google-analytics/adwords_conversion.html", takes_context=True)(google_adwords_signup)



from datetime import datetime
from decimal import Decimal
from django.template import loader, Context
from django.utils.http import urlencode
from django.utils.translation import ugettext_lazy as _
from payment.modules.base import BasePaymentProcessor, ProcessorResult, NOTSET
from satchmo_store.shop.models import Config
from satchmo_utils.numbers import trunc_decimal
from tax.utils import get_tax_processor
from xml.dom import minidom
import random
import urllib2

class PaymentProcessor(BasePaymentProcessor):
    """
    Authorize.NET payment processing module
    You must have an account with authorize.net in order to use this module.
    
    Additionally, you must have ARB enabled in your account to use recurring billing.
    """
    def __init__(self, settings):
        super(PaymentProcessor, self).__init__('authorizenet', settings)
        self.arb_enabled = settings.ARB.value

    def authorize_payment(self, order=None, amount=NOTSET, testing=False):
        """Authorize a single payment.
        
        Returns: ProcessorResult
        """
        if order:
            self.prepare_data(order)
        else:
            order = self.order
            
        if order.paid_in_full:
            self.log_extra('%s is paid in full, no authorization attempted.', order)
            results = ProcessorResult(self.key, True, _("No charge needed, paid in full."))
        else:
            self.log_extra('Authorizing payment of %s for %s', amount, order)

            standard = self.get_standard_charge_data(authorize=True, amount=amount)
            results = self.send_post(standard, testing)

        return results

    def can_authorize(self):
        return True

    def can_recur_bill(self):
        return True

    def capture_authorized_payment(self, authorization, testing=False, order=None, amount=NOTSET):
        """Capture a single payment"""
        if order:
            self.prepare_data(order)
        else:
            order = self.order

        if order.authorized_remaining == Decimal('0.00'):
            self.log_extra('No remaining authorizations on %s', order)
            return ProcessorResult(self.key, True, _("Already complete"))

        self.log_extra('Capturing Authorization #%i for %s', authorization.id, order)
        data = self.get_prior_auth_data(authorization, amount=amount)
        results = None
        if data:
            results = self.send_post(data, testing)
        
        return results
        
    def capture_payment(self, testing=False, order=None, amount=NOTSET):
        """Process payments without an authorization step."""
        if order:
            self.prepare_data(order)
        else:
            order = self.order

        recurlist = self.get_recurring_charge_data()
        if recurlist:
            success, results = self.process_recurring_subscriptions(recurlist, testing)
            if not success:
                self.log_extra('recur payment failed, aborting the rest of the module')
                return results

        if order.paid_in_full:
            self.log_extra('%s is paid in full, no capture attempted.', order)
            results = ProcessorResult(self.key, True, _("No charge needed, paid in full."))
            self.record_payment()
        else:
            self.log_extra('Capturing payment for %s', order)
            
            standard = self.get_standard_charge_data(amount=amount)
            results = self.send_post(standard, testing)
            
        return results

    def get_prior_auth_data(self, authorization, amount=NOTSET):
        """Build the dictionary needed to process a prior auth capture."""
        settings = self.settings
        trans = {'authorization' : authorization}
        remaining = authorization.remaining()
        if amount == NOTSET or amount > remaining:
            amount = remaining
        
        balance = trunc_decimal(amount, 2)
        trans['amount'] = amount
        
        if self.is_live():
            conn = settings.CONNECTION.value
            self.log_extra('Using live connection.')
        else:
            testflag = 'TRUE'
            conn = settings.CONNECTION_TEST.value
            self.log_extra('Using test connection.')
            
        if self.settings.SIMULATE.value:
            testflag = 'TRUE'
        else:
            testflag = 'FALSE'

        trans['connection'] = conn

        trans['configuration'] = {
            'x_login' : settings.LOGIN.value,
            'x_tran_key' : settings.TRANKEY.value,
            'x_version' : '3.1',
            'x_relay_response' : 'FALSE',
            'x_test_request' : testflag,
            'x_delim_data' : 'TRUE',
            'x_delim_char' : '|',
            'x_type': 'PRIOR_AUTH_CAPTURE',
            'x_trans_id' : authorization.transaction_id
            }
        
        self.log_extra('prior auth configuration: %s', trans['configuration'])
                
        trans['transactionData'] = {
            'x_amount' : balance,
            }

        part1 = urlencode(trans['configuration']) 
        postdata = part1 + "&" + urlencode(trans['transactionData'])
        trans['postString'] = postdata
        
        self.log_extra('prior auth poststring: %s', postdata)
        trans['logPostString'] = postdata
        
        return trans
        
        
    def get_void_auth_data(self, authorization):
        """Build the dictionary needed to process a prior auth release."""
        settings = self.settings
        trans = {
            'authorization' : authorization,
            'amount' : Decimal('0.00'),
        }

        if self.is_live():
            conn = settings.CONNECTION.value
            self.log_extra('Using live connection.')
        else:
            testflag = 'TRUE'
            conn = settings.CONNECTION_TEST.value
            self.log_extra('Using test connection.')

        if self.settings.SIMULATE.value:
            testflag = 'TRUE'
        else:
            testflag = 'FALSE'

        trans['connection'] = conn

        trans['configuration'] = {
            'x_login' : settings.LOGIN.value,
            'x_tran_key' : settings.TRANKEY.value,
            'x_version' : '3.1',
            'x_relay_response' : 'FALSE',
            'x_test_request' : testflag,
            'x_delim_data' : 'TRUE',
            'x_delim_char' : '|',
            'x_type': 'VOID',
            'x_trans_id' : authorization.transaction_id
            }

        self.log_extra('void auth configuration: %s', trans['configuration'])

        postdata = urlencode(trans['configuration']) 
        trans['postString'] = postdata

        self.log_extra('void auth poststring: %s', postdata)
        trans['logPostString'] = postdata

        return trans

    def get_recurring_charge_data(self, testing=False):
        """Build the list of dictionaries needed to process a recurring charge.
        
        Because Authorize can only take one subscription at a time, we build a list
        of the transaction dictionaries, for later sequential posting.
        """
        if not self.arb_enabled:
            return []
        
        # get all subscriptions from the order
        subscriptions = self.get_recurring_orderitems()
        
        if len(subscriptions) == 0:
            self.log_extra('No subscription items')
            return []

        settings = self.settings            
        # set up the base dictionary
        trans = {}

        if self.is_live():
            conn = settings.ARB_CONNECTION.value
            self.log_extra('Using live recurring charge connection.')
        else:
            conn = settings.ARB_CONNECTION_TEST.value
            self.log_extra('Using test recurring charge connection.')
        
        shop_config = Config.objects.get_current()
        
        trans['connection'] = conn
        trans['config'] = {
            'merchantID' : settings.LOGIN.value,
            'transactionKey' : settings.TRANKEY.value,
            'shop_name' : shop_config.store_name,
        }
        trans['order'] = self.order
        trans['card'] = self.order.credit_card
        trans['card_expiration'] =  "%4i-%02i" % (self.order.credit_card.expire_year, self.order.credit_card.expire_month)
        
        translist = []
        taxer = get_tax_processor(user = self.order.contact.user)
        
        for subscription in subscriptions:
            product = subscription.product
            subtrans = trans.copy()
            subtrans['subscription'] = subscription
            subtrans['product'] = product
            
            sub = product.subscriptionproduct
            
            trial = sub.get_trial_terms(0)
            if trial:
                price = trunc_decimal(trial.price, 2)
                trial_amount = price
                if price and subscription.product.taxable:
                    trial_amount = taxer.by_price(subscription.product.taxClass, price)
                    #todo, maybe add shipping for trial?
                amount = sub.recurring_price()
                trial_occurrences = trial.occurrences
                if not trial_occurrences:
                    self.log.warn("Trial expiration period is less than one recurring billing cycle. " +
                        "Authorize does not allow this, so the trial period has been adjusted to be equal to one recurring cycle.")
                    trial_occurrences = 1
            else:
                trial_occurrences = 0
                trial_amount = Decimal('0.00')
                amount = subscription.total_with_tax

            occurrences = sub.recurring_times + trial_occurrences
            if occurrences > 9999:
                occurrences = 9999

            subtrans['occurrences'] = occurrences
            subtrans['trial_occurrences'] = trial_occurrences
            subtrans['trial'] = trial
            subtrans['trial_amount'] = trunc_decimal(trial_amount, 2)
            subtrans['amount'] = trunc_decimal(amount, 2)
            if trial:
                charged_today = trial_amount
            else:
                charged_today = amount
            
            charged_today = trunc_decimal(charged_today, 2)
                
            subtrans['charged_today'] = charged_today
            translist.append(subtrans)
            
        return translist
        
    def get_standard_charge_data(self, amount=NOTSET, authorize=False):
        """Build the dictionary needed to process a credit card charge"""

        order = self.order
        settings = self.settings
        trans = {}
        if amount == NOTSET:
            amount = order.balance
            
        balance = trunc_decimal(amount, 2)
        trans['amount'] = balance
        
        if self.is_live():
            conn = settings.CONNECTION.value
            self.log_extra('Using live connection.')
        else:
            testflag = 'TRUE'
            conn = settings.CONNECTION_TEST.value
            self.log_extra('Using test connection.')
            
        if self.settings.SIMULATE.value:
            testflag = 'TRUE'
        else:
            testflag = 'FALSE'

        trans['connection'] = conn
            
        trans['authorize_only'] = authorize

        if not authorize:
            transaction_type = 'AUTH_CAPTURE'
        else:
            transaction_type = 'AUTH_ONLY'
                        
        trans['configuration'] = {
            'x_login' : settings.LOGIN.value,
            'x_tran_key' : settings.TRANKEY.value,
            'x_version' : '3.1',
            'x_relay_response' : 'FALSE',
            'x_test_request' : testflag,
            'x_delim_data' : 'TRUE',
            'x_delim_char' : '|',
            'x_type': transaction_type,
            'x_method': 'CC',
            }
        
        self.log_extra('standard charges configuration: %s', trans['configuration'])
        
        trans['custBillData'] = {
            'x_first_name' : order.contact.first_name,
            'x_last_name' : order.contact.last_name,
            'x_address': order.full_bill_street,
            'x_city': order.bill_city,
            'x_state' : order.bill_state,
            'x_zip' : order.bill_postal_code,
            'x_country': order.bill_country,
            'x_phone' : order.contact.primary_phone.phone,
            'x_email' : order.contact.email,
            }
    
        self.log_extra('standard charges configuration: %s', trans['custBillData'])
        
        invoice = "%s" % order.id
        failct = order.paymentfailures.count()
        if failct > 0:
            invoice = "%s_%i" % (invoice, failct)

        if not self.is_live():
            # add random test id to this, for testing repeatability
            invoice = "%s_test_%s_%i" % (invoice,  datetime.now().strftime('%m%d%y'), random.randint(1,1000000))
        
        cc = order.credit_card.decryptedCC
        ccv = order.credit_card.ccv
        if not self.is_live() and cc == '4222222222222':
            if ccv == '222':
                self.log_extra('Setting a bad ccv number to force an error')
                ccv = '1'
            else:
                self.log_extra('Setting a bad credit card number to force an error')
                cc = '1234'
        trans['transactionData'] = {
            'x_amount' : balance,
            'x_card_num' : cc,
            'x_exp_date' : order.credit_card.expirationDate,
            'x_card_code' : ccv,
            'x_invoice_num' : invoice
            }

        part1 = urlencode(trans['configuration']) + "&"
        part2 = "&" + urlencode(trans['custBillData'])
        trans['postString'] = part1 + urlencode(trans['transactionData']) + part2
        
        redactedData = {
            'x_amount' : balance,
            'x_card_num' : order.credit_card.display_cc,
            'x_exp_date' : order.credit_card.expirationDate,
            'x_card_code' : "REDACTED",
            'x_invoice_num' : invoice
        }
        self.log_extra('standard charges transactionData: %s', redactedData)
        trans['logPostString'] = part1 + urlencode(redactedData) + part2
        
        return trans
        
    def process_recurring_subscriptions(self, recurlist, testing=False):
        """Post all subscription requests."""    
        
        results = []
        for recur in recurlist:
            success, reason, response, subscription_id = self.process_recurring_subscription(recur, testing=testing)
            if success:
                if not testing:
                    payment = self.record_payment(order=self.order, amount=recur['charged_today'], transaction_id=subscription_id, reason_code=reason)
                    results.append(ProcessorResult(self.key, success, response, payment=payment))
            else:
                self.log.info("Failed to process recurring subscription, %s: %s", reason, response)
                break
        
        return success, results
        
    def process_recurring_subscription(self, data, testing=False):
        """Post one subscription request."""
        self.log_extra('Processing subscription: %s', data['product'].slug)
        
        t = loader.get_template('shop/checkout/authorizenet/arb_create_subscription.xml')
        ctx = Context(data)
        request = t.render(ctx)
        
        if self.settings.EXTRA_LOGGING.value:
            data['redact'] = True
            ctx = Context(data)
            redacted = t.render(ctx)
            self.log_extra('Posting data to: %s\n%s', data['connection'], redacted)
        
        headers = {'Content-type':'text/xml'}
        conn = urllib2.Request(data['connection'], request, headers)
        try:
            f = urllib2.urlopen(conn)
            all_results = f.read()
        except urllib2.URLError, ue:
            self.log.error("error opening %s\n%s", data['connection'], ue)
            return (False, 'ERROR', _('Could not talk to Authorize.net gateway'), None)
        
        self.log_extra('Authorize response: %s', all_results)
        
        subscriptionID = None
        try:
            response = minidom.parseString(all_results)
            doc = response.documentElement
            reason = doc.getElementsByTagName('code')[0].firstChild.nodeValue
            response_text = doc.getElementsByTagName('text')[0].firstChild.nodeValue                
            result = doc.getElementsByTagName('resultCode')[0].firstChild.nodeValue
            success = result == "Ok"

            if success:
                #refID = doc.getElementsByTagName('refId')[0].firstChild.nodeValue
                subscriptionID = doc.getElementsByTagName('subscriptionId')[0].firstChild.nodeValue
        except Exception, e:
            self.log.error("Error %s\nCould not parse response: %s", e, all_results)
            success = False
            reason = "Parse Error"
            response_text = "Could not parse response"
            
        return success, reason, response_text, subscriptionID
        
        
    def release_authorized_payment(self, order=None, auth=None, testing=False):
        """Release a previously authorized payment."""
        if order:
            self.prepare_data(order)
        else:
            order = self.order

        self.log_extra('Releasing Authorization #%i for %s', auth.id, order)
        data = self.get_void_auth_data(auth)
        results = None
        if data:
            results = self.send_post(data, testing)
            
        if results.success:
            auth.complete = True
            auth.save()
            
        return results
        
    def send_post(self, data, testing=False, amount=NOTSET):
        """Execute the post to Authorize Net.
        
        Params:
        - data: dictionary as returned by get_standard_charge_data
        - testing: if true, then don't record the payment
        
        Returns:
        - ProcessorResult
        """
        self.log.info("About to send a request to authorize.net: %(connection)s\n%(logPostString)s", data)

        conn = urllib2.Request(url=data['connection'], data=data['postString'])
        try:
            f = urllib2.urlopen(conn)
            all_results = f.read()
            self.log_extra('Authorize response: %s', all_results)
        except urllib2.URLError, ue:
            self.log.error("error opening %s\n%s", data['connection'], ue)
            return ProcessorResult(self.key, False, _('Could not talk to Authorize.net gateway'))
            
        parsed_results = all_results.split(data['configuration']['x_delim_char'])
        response_code = parsed_results[0]
        reason_code = parsed_results[1]
        response_text = parsed_results[3]
        transaction_id = parsed_results[6]
        success = response_code == '1'
        if amount == NOTSET:
            amount = data['amount']

        payment = None
        if success and not testing:
            if data.get('authorize_only', False):
                self.log_extra('Success, recording authorization')
                payment = self.record_authorization(order=self.order, amount=amount, 
                    transaction_id=transaction_id, reason_code=reason_code)
            else:
                if amount <= 0:
                    self.log_extra('Success, recording refund')
                else:
                    self.log_extra('Success, recording payment')
                authorization = data.get('authorization', None)
                payment = self.record_payment(order=self.order, amount=amount, 
                    transaction_id=transaction_id, reason_code=reason_code, authorization=authorization)
            
        elif not testing:
            payment = self.record_failure(amount=amount, transaction_id=transaction_id, 
                reason_code=reason_code, details=response_text)

        self.log_extra("Returning success=%s, reason=%s, response_text=%s", success, reason_code, response_text)
        return ProcessorResult(self.key, success, response_text, payment=payment)

if __name__ == "__main__":
    """
    This is for testing - enabling you to run from the command line and make
    sure everything is ok
    """
    import os
    from livesettings import config_get_group

    # Set up some dummy classes to mimic classes being passed through Satchmo
    class testContact(object):
        pass
    class testCC(object):
        pass
    class testOrder(object):
        def __init__(self):
            self.contact = testContact()
            self.credit_card = testCC()
        def order_success(self):
            pass

    if not os.environ.has_key("DJANGO_SETTINGS_MODULE"):
        os.environ["DJANGO_SETTINGS_MODULE"]="satchmo_store.settings"

    settings_module = os.environ['DJANGO_SETTINGS_MODULE']
    settingsl = settings_module.split('.')
    settings = __import__(settings_module, {}, {}, settingsl[-1])

    sampleOrder = testOrder()
    sampleOrder.contact.first_name = 'Chris'
    sampleOrder.contact.last_name = 'Smith'
    sampleOrder.contact.primary_phone = '801-555-9242'
    sampleOrder.full_bill_street = '123 Main Street'
    sampleOrder.bill_postal_code = '12345'
    sampleOrder.bill_state = 'TN'
    sampleOrder.bill_city = 'Some City'
    sampleOrder.bill_country = 'US'
    sampleOrder.total = "27.01"
    sampleOrder.balance = "27.01"
    sampleOrder.credit_card.decryptedCC = '6011000000000012'
    sampleOrder.credit_card.expirationDate = "10/11"
    sampleOrder.credit_card.ccv = "144"

    authorize_settings = config_get_group('PAYMENT_AUTHORIZENET')
    if authorize_settings.LIVE.value:
        print "Warning.  You are submitting a live order.  AUTHORIZE.NET system is set LIVE."
        
    processor = PaymentProcessor(authorize_settings)
    processor.prepare_data(sampleOrder)
    results = processor.process(testing=True)
    print results





"""Context menus with options to edit menus"""
# Copyright (C) 2006 by Nokia Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
import gtk

from gazpacho.actioneditor import CommandAddRemoveAction, CommandEditAction
from gazpacho.commandmanager import command_manager
from gazpacho.i18n import _
from gazpacho.stockicons import StockIconDialog

class ContextMenu(gtk.Menu):
    """Base class for context menus for MenuBar and Menu widgets"""

    def __init__(self, context):
        """Constructor.

        Context must be the MenuItem instance where the  user clicked
        with the right button
        """
        gtk.Menu.__init__(self)

        self.context = context

        self._create_options()
        self.show_all()

    def _create_options(self):
        """Create the menu items. Subclasses should override this method"""

    def _on_edit__activate(self, item):
        """Edit item callback"""
        self.context.start_editing()

    def _on_previous__activate(self, item):
        """Previous item callback

        Moves the context one position before where it was.
        """
        parent = self.context.parent
        i = parent.children.index(self.context)
        parent.remove(self.context)
        parent.insert(self.context, i - 1)

        menu_bar = self.context.get_menu_bar()
        menu_bar.update_ui()

    def _on_next__activate(self, item):
        """Next item callback.

        Moves the context one position after where it was.
        """
        parent = self.context.parent
        i = parent.children.index(self.context)
        parent.remove(self.context)
        parent.insert(self.context, i + 1)

        menu_bar = self.context.get_menu_bar()
        menu_bar.update_ui()

    def _on_delete__activate(self, item):
        """Delete item callback.

        Removes the context from its parent.
        """
        gadget = self.context.gadget
        if gadget:
            action_group = self.context.action_group
            action = self.context.action
            cmd = CommandAddRemoveAction(action_group, action, False)
            project = gadget.project
            command_manager.execute(cmd, project)

        menu_bar = self.context.get_menu_bar()
        self.context.parent.remove(self.context)
        menu_bar.update_ui()

class MenuBarContextMenu(ContextMenu):
    """Context menu for MenuBar widgets"""

    def _create_options(self):
        """Creates edit, left, right and delete items"""
        edit = gtk.ImageMenuItem(gtk.STOCK_EDIT)
        edit.connect('activate', self._on_edit__activate)
        self.append(edit)

        self.append(gtk.SeparatorMenuItem())

        go_somewhere = False

        if not self.context.is_first():
            # The first item can not be moved to the left
            move_left = gtk.ImageMenuItem(gtk.STOCK_GO_BACK)
            move_left.connect('activate', self._on_previous__activate)
            self.append(move_left)
            go_somewhere = True

        if not self.context.is_last():
            # The last item can not be moved to the right
            move_right = gtk.ImageMenuItem(gtk.STOCK_GO_FORWARD)
            move_right.connect('activate', self._on_next__activate)
            self.append(move_right)
            go_somewhere = True

        if go_somewhere:
            self.append(gtk.SeparatorMenuItem())

        delete = gtk.ImageMenuItem(gtk.STOCK_DELETE)
        delete.connect('activate', self._on_delete__activate)
        self.append(delete)

class MenuContextMenu(ContextMenu):
    """Context menu for Menu widgets"""

    def _create_options(self):
        """Creates edit, up, bottom, new group, add/remove submenu, set image,
        set accelerator and delete items"""
        edit = gtk.ImageMenuItem(gtk.STOCK_EDIT)
        edit.connect('activate', self._on_edit__activate)
        self.append(edit)

        self.append(gtk.SeparatorMenuItem())

        go_somewhere = False

        if not self.context.is_first():
            # The first item can not be moved up
            move_up = gtk.ImageMenuItem(gtk.STOCK_GO_UP)
            move_up.connect('activate', self._on_previous__activate)
            self.append(move_up)
            go_somewhere = True

        if not self.context.is_last():
            # The last item can not be moved down
            move_down = gtk.ImageMenuItem(gtk.STOCK_GO_DOWN)
            move_down.connect('activate', self._on_next__activate)
            self.append(move_down)
            go_somewhere = True

        if go_somewhere:
            self.append(gtk.SeparatorMenuItem())

        # A new group has a separator just before it
        new_group = self.context.is_new_group()
        start_new_group = gtk.CheckMenuItem(_('Start new group'))
        start_new_group.set_active(new_group)
        start_new_group.connect('toggled',
                                self._on_start_new_group__toggled)
        self.append(start_new_group)

        if self.context.submenu:
            # let the user remove the submenu
            submenu = gtk.MenuItem(_('Remove submenu'))
            submenu.connect('activate', self._on_remove_submenu__activate)
        else:
            # or add a new one
            submenu = gtk.MenuItem(_('Add submenu'))
            submenu.connect('activate', self._on_add_submenu__activate)

        self.append(submenu)

        self.append(gtk.SeparatorMenuItem())

        # Let the user set a stock image
        image = gtk.MenuItem(_('Set image'))
        image.connect('activate', self._on_image__activate)
        self.append(image)

        # Or an accelerator ( TODO )
        #accel = gtk.MenuItem(_('Set accelerator'))
        #accel.connect('activate', self._on_accel__activate)
        #self.append(accel)

        self.append(gtk.SeparatorMenuItem())

        delete = gtk.ImageMenuItem(gtk.STOCK_DELETE)
        delete.connect('activate', self._on_delete__activate)
        self.append(delete)

    def _on_start_new_group__toggled(self, item):
        """Start new group item callback.

        Add or remove a separator before the context depending on the
        previous existence of the separator
        """
        from . import widgets
        parent = self.context.parent
        i = parent.children.index(self.context)

        if item.get_active() and not self.context.is_new_group():
            # Add a separator before the context
            separator = widgets.SeparatorMenuItem()
            separator.show()
            parent.insert(separator, i)

        elif not item.get_active() and self.context.is_new_group():
            # remove the separator before the context
            separator = parent.children[i - 1]
            parent.remove(separator)

        # create the UI definition
        menu_bar = self.context.get_menu_bar()
        menu_bar.update_ui()

    def _on_add_submenu__activate(self, item):
        """Add submenu item callback.

        Add a submenu on the context
        """
        from . import widgets
        submenu = widgets.Menu(self.context.parent.gadget)
        self.context.set_submenu(submenu)

    def _on_remove_submenu__activate(self, item):
        """Remove submenu item callback

        Removes the submenu of the context
        """
        if self.context.submenu:
            self.context.submenu.popdown()
            self.context.set_submenu(None)

    def _on_image__activate(self, item):
        """Image item callback

        Pop up a dialog with the stock icons so the user can select one of
        those. TODO: let the user reset the image to None
        """
        dialog = StockIconDialog()
        if gtk.RESPONSE_OK == dialog.run():
            stock_id = dialog.get_selected()

            # Update the action
            action = self.context.action
            project = self.context.gadget.project
            values = {
                'name' : action.name,
                'label': action.label,
                'short_label': action.short_label,
                'is_important': action.is_important,
                'stock_id': stock_id,
                'tooltip': action.tooltip,
                'accelerator': action.accelerator,
                'callback': action.callback,
                }
            cmd = CommandEditAction(action, values, project)
            command_manager.execute(cmd, project)

            self.context.set_action(action)

        dialog.destroy()



from decimal import Decimal, ROUND_HALF_UP, InvalidOperation
from django.conf import settings
import datetime
import logging
import os
import random
import sys
import types

log = logging.getLogger('shop.utils')

def add_month(date, n=1):
    """add n+1 months to date then subtract 1 day
    to get eom, last day of target month"""
    oneday = datetime.timedelta(days=1)
    q,r = divmod(date.month+n, 12)
    eom = datetime.date(date.year+q, r+1, 1) - oneday
    if date.month != (date+oneday).month or date.day >= eom.day:
        return eom
    return eom.replace(day=date.day)

def app_enabled(appname):
    """Check the app list to see if a named app is installed."""
    from django.db import models
    
    all_apps = {}
    for app in models.get_apps():
        n = app.__name__.split('.')[-2]
        if n  == appname:
            return True
    return False

def can_loop_over(maybe):
    """Test value to see if it is list like"""
    try:
        iter(maybe)
    except:
        return 0
    else:
        return 1

def cross_list(sequences):
    """
    Code taken from the Python cookbook v.2 (19.9 - Looping through the cross-product of multiple iterators)
    This is used to create all the variations associated with an product
    """
    result =[[]]
    for seq in sequences:
        result = [sublist+[item] for sublist in result for item in seq]
    return result

def current_media_url(request):    
    """Return the media_url, taking into account SSL."""
    media_url = settings.MEDIA_URL
    secure = request_is_secure(request)
    if secure:
        try:
            media_url = settings.MEDIA_SECURE_URL
        except AttributeError:
            media_url = media_url.replace('http://','https://')
    return media_url

def is_scalar(maybe):
    """Test to see value is a string, an int, or some other scalar type"""
    return is_string_like(maybe) or not can_loop_over(maybe)

def flatten_list(sequence, scalarp=is_scalar, result=None):
    """flatten out a list by putting sublist entries in the main list"""
    if result is None:
        result = []

    for item in sequence:
        if scalarp(item):
            result.append(item)
        else:
            flatten_list(item, scalarp, result)

def flatten(sequence, scalarp=is_scalar):
    """flatten out a list by putting sublist entries in the main list"""
    for item in sequence:
        if scalarp(item):
            yield item
        else:
            for subitem in flatten(item, scalarp):
                yield subitem

def get_flat_list(sequence):
    """flatten out a list and return the flat list"""
    flat = []
    flatten_list(sequence, result=flat)
    return flat

def is_list_or_tuple(maybe):
    return isinstance(maybe, (types.TupleType, types.ListType))

def is_string_like(maybe):
    """Test value to see if it acts like a string"""
    try:
        maybe+""
    except TypeError:
        return 0
    else:
        return 1

def load_module(module):
    """Load a named python module."""
    try:
        module = sys.modules[module]
    except KeyError:
        __import__(module)
        module = sys.modules[module]
    return module

_MODULES = []
def load_once(key, module):
    if key not in _MODULES:
        load_module(module)
        _MODULES.append(key)

def normalize_dir(dir_name):
    if not dir_name.startswith('./'):
        dir_name = url_join('.', dir_name)
    if dir_name.endswith("/"):
        dir_name = dir_name[:-1]
    return dir_name

_LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

def random_string(length, variable=False, charset=_LETTERS):
    if variable:
        length = random.randrange(1, length+1)
    return ''.join([random.choice(charset) for x in xrange(length)])

def request_is_secure(request):
    if request.is_secure():
        return True

    # Handle forwarded SSL (used at Webfaction)
    if 'HTTP_X_FORWARDED_SSL' in request.META:
        return request.META['HTTP_X_FORWARDED_SSL'] == 'on'

    # Handle an additional case of proxying SSL requests. This is useful for Media Temple's
    # Django container
    if 'HTTP_X_FORWARDED_HOST' in request.META and request.META['HTTP_X_FORWARDED_HOST'].endswith('443'):
        return True

    return False
            
def url_join(*args):
    """Join any arbitrary strings into a forward-slash delimited string.
    Do not strip leading / from first element, nor trailing / from last element.

    This function can take lists as arguments, flattening them appropriately.

    example:
    url_join('one','two',['three','four'],'five') => 'one/two/three/four/five'
    """
    if len(args) == 0:
        return ""

    args = get_flat_list(args)

    if len(args) == 1:
        return str(args[0])

    else:
        args = [str(arg).replace("\\", "/") for arg in args]

        work = [args[0]]
        for arg in args[1:]:
            if arg.startswith("/"):
                work.append(arg[1:])
            else:
                work.append(arg)

        joined = reduce(os.path.join, work)

    return joined.replace("\\", "/")



"""
USPS Shipping Module
You must have a USPS account to use this module.
You may register at usps.com

This module uses the XML online tools for maximum flexibility.  It is
primarily created for use in the US but reconfiguring for international
shipping or more advanced uses should be straightforward.

It is recommended that you refer to the USPS shipper developer documents
(available when you register at USPS) in order to tailor this to your
unique needs.
"""

# Note, make sure you use decimal math everywhere!
from decimal import Decimal
from django.core.cache import cache
from django.template import Context, loader
from django.utils.translation import ugettext as _
from l10n.models import Country
from livesettings import config_get_group, config_value
from shipping.modules.base import BaseShipper
import logging
import urllib2
try:
    from xml.etree.ElementTree import fromstring, tostring
except ImportError:
    from elementtree.ElementTree import fromstring, tostring

"""
The different class codes for each type of mail.  Some types of mail have
several sub-options.
"""
CODES = {'0': 'FIRST CLASS',
         '1': 'PRIORITY',
         '16': 'PRIORITY',
         '17': 'PRIORITY',
         '22': 'PRIORITY',
         '3': 'EXPRESS',
         '13': 'EXPRESS',
         '4': 'PARCEL',
         '5': 'BPM',
         '6': 'MEDIA',
         '7': 'LIBRARY',
         
         # these are here to avoid KeyErrors
         '2': 'INTL',
         '8': 'INTL',
         '9': 'INTL',
         '10': 'INTL',
         '11': 'INTL',
         '12': 'INTL',
         '14': 'INTL',
         '15': 'INTL',
        }
        
"""
International service codes

        (('14', 'First Class Mail International Large Envelope')),
        (('15', 'First Class Mail International Package')),
        (('1', 'Express Mail International (EMS)')),
        (('4', 'Global Express Guaranteed')),
        (('6', 'Global Express Guaranteed Non-Document Rectangular')),
        (('7', 'Global Express Guaranteed Non-Document Non-Rectangular')),
        (('10', 'Express Mail International (EMS) Flat-Rate Envelope')),
        (('2', 'Priority Mail International')),
        (('8', 'Priority Mail International Flat-Rate Envelope')),
        (('9', 'Priority Mail International Flat-Rate Box')),
        (('11', 'Priority Mail International Large Flat-Rate Box')),
        (('12', 'USPS GXG Envelopes')),
"""

"""
These are the deliver estimates from the website at
http://www.usps.com/webtools/htm/DomMailServStandv1-0.htm#_Toc82841084
"""
ESTIMATES = {'FIRST CLASS': '3 - 5',
             'PRIORITY': '1 - 3',
             'EXPRESS': '1 - 2',
             'PARCEL': '2 - 9',
             'BPM': '2 - 9',
             'MEDIA': '2 - 9',
             'LIBRARY': '2 - 9',
             'INTL': '',
            }

"""
A list of API's for each type of mail service
"""
APIS = {'PRIORITY': 'PriorityMail',
        'EXPRESS': 'ExpressMailCommitment',
        'PARCEL': 'StandardB',
        'BPM': 'StandardB',
        'MEDIA': 'StandardB',
        'LIBRARY': 'StandardB',
        'INTL': '',
       }

log = logging.getLogger('usps.shipper')
class Shipper(BaseShipper):

    def __init__(self, cart=None, contact=None, service_type=None):
        self._calculated = False
        self.cart = cart
        self.contact = contact
        if service_type:
            self.service_type_code = service_type[0]
            self.service_type_text = service_type[1]
        else:
            self.service_type_code = "99"
            self.service_type_text = "Uninitialized"
        self.id = u"USPS-%s-%s" % (self.service_type_code, self.service_type_text)
        self.raw = "NO DATA"
        self.exact_date = False
        #if cart or contact:
        #    self.calculate(cart, contact)

    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "U.S. Postal Service"

    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        return _("USPS - %s" % self.service_type_text.replace('Int`l: ', ''))

    def cost(self):
        """
        Complex calculations can be done here as long as the return value is a decimal figure
        """
        assert(self._calculated)
        settings =  config_get_group('shipping.modules.usps')
        if settings.HANDLING_FEE and float(str(settings.HANDLING_FEE)) > 0.0:
            self.charges = Decimal(self.charges) + Decimal(str(settings.HANDLING_FEE))
        return Decimal(str(self.charges))

    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        return _("USPS")

    def expectedDelivery(self):
        """
        Can be a plain string or complex calcuation returning an actual date
        """
        if self.exact_date:
            if self.is_intl:
                return _('in %s' % self.delivery_days.lower())
            return _("by %s" % self.delivery_days.replace('-', ' '))
        elif self.delivery_days != "1":
            return _("in %s business days" % self.delivery_days)
        else:
            return _("in %s business day" % self.delivery_days)

    def valid(self, order=None):
        """
        Can do complex validation about whether or not this option is valid.
        For example, may check to see if the recipient is in an allowed country
        or location.
        """
        return self.is_valid

    def _process_request(self, connection, request, api=None):
        """
        Post the data and return the XML response
        """
        # determine which API to use
        if api == None:
            if self.is_intl:
                api = 'IntlRate'
            else:
                api = 'RateV3'

        data = 'API=%s&XML=%s' % (api, request.encode('utf-8'))

        conn = urllib2.Request(url=connection, data=data)
        f = urllib2.urlopen(conn)
        all_results = f.read()

        log.error(all_results)

        return (fromstring(all_results))

    def render_template(self, template, cart=None, contact=None):
        from satchmo_store.shop.models import Config
        shop_details = Config.objects.get_current()
        settings =  config_get_group('shipping.modules.usps')

        if not self.is_intl:
            mail_type = CODES[self.service_type_code]
            if mail_type == 'INTL': return ''
        
            if mail_type == 'FIRST CLASS':
                self.api = None
            else:
                self.api = APIS[mail_type]
        else:
            mail_type = None
            self.api = None
        
        # calculate the weight of the entire order
        weight = Decimal('0.0')
        for item in cart.cartitem_set.all():
            if item.product.smart_attr('weight'):
                weight += item.product.smart_attr('weight') * item.quantity
        self.verbose_log('WEIGHT: %s' % weight)

        # I don't know why USPS made this one API different this way...
        if self.api == 'ExpressMailCommitment':
            zip_ending = 'ZIP'
        else:
            zip_ending = 'zip'

        # get the shipping country (for the international orders)
        ship_country = contact.shipping_address.country.printable_name

        configuration = {
            'userid': settings.USER_ID.value,
            'password': settings.USER_PASSWORD.value,
            'container': settings.SHIPPING_CONTAINER.value,
            'ship_type': mail_type,
            'shop_details': shop_details
        }
        c = Context({
                'config': configuration,
                'cart': cart,
                'contact': contact,
                'is_international': self.is_intl,
                'api': self.api,
                'weight': weight,
                'zip': zip_ending,
                'country': ship_country,
                'first_class_types': ['LETTER', 'FLAT', 'PARCEL']
        })
        t = loader.get_template(template)
        return t.render(c)

    def calculate(self, cart, contact):
        """
        Based on the chosen USPS method, we will do our call to USPS and see how
        much it will cost. We will also need to store the results for further 
        parsing and return via the methods above
        """
        from satchmo_store.shop.models import Config
        settings =  config_get_group('shipping.modules.usps')
        shop_details = Config.objects.get_current()
        self.is_intl = contact.shipping_address.country.iso2_code != shop_details.country.iso2_code
        self.delivery_days = ESTIMATES[CODES[self.service_type_code]]

        if not self.is_intl:
            template = 'shipping/usps/request.xml'
        else:
            template = 'shipping/usps/request_intl.xml'
        request = self.render_template(template, cart, contact)
        self.is_valid = False

        if settings.LIVE.value:
            connection = settings.CONNECTION.value
        else:
            connection = settings.CONNECTION_TEST.value

        cache_key_response = "usps-cart-%s-response" % int(cart.id)
        cache_key_request = "usps-cart-%s-request" % int(cart.id)
        last_request = cache.get(cache_key_request)

        tree = cache.get(cache_key_response)

        if (last_request != request) or tree is None:
            self.verbose_log("Requesting from USPS [%s]\n%s", cache_key_request, request)
            cache.set(cache_key_request, request, 60)
            tree = self._process_request(connection, request)
            self.verbose_log("Got from USPS [%s]:\n%s", cache_key_response, self.raw)
            needs_cache = True
        else:
            needs_cache = False

        errors = tree.getiterator('Error')

        # if USPS returned no error, return the prices
        if errors == None or len(errors) == 0:
            # check for domestic results first
            all_packages = tree.getiterator('RateV3Response')

            # if there are none, revert to international results
            if len(all_packages) == 0:
                all_packages = tree.getiterator('IntlRateResponse')
                for package in all_packages:
                    for service in package.getiterator('Service'):
                        #self.verbose_log('%s vs %s' % (service.attrib['ID'], self.service_type_code))
                        if service.attrib['ID'] == self.service_type_code and \
                            self.service_type_text.startswith('Int`l: '):
                            
                            self.charges = service.find('.//Postage/').text
                            self.delivery_days = service.find('.//SvcCommitments/').text
                            #self.verbose_log('%s %s' % (self.charges, self.delivery_days))
                            self.is_valid = True
                            self._calculated = True
                            self.exact_date = True

                            #if needs_cache:
                            #    cache.set(cache_key_response, tree, 60)
            else:
                for package in all_packages:
                    for postage in package.getiterator('Postage'):
                        if postage.attrib['CLASSID'] == self.service_type_code and \
                            not self.service_type_text.startswith('Int`l: '):
                            self.charges = postage.find('.//Rate/').text

                            # Now try to figure out how long it would take for this delivery
                            if self.api:
                                delivery = self.render_template('shipping/usps/delivery.xml', cart, contact)
                                del_tree = self._process_request(connection, delivery, self.api)
                                parent = '%sResponse' % self.api
                                del_iter = del_tree.getiterator(parent)

                                if len(del_iter):
                                    i = del_iter[0]

                                    # express mail usually has a date commitment
                                    if self.api == 'ExpressMailCommitment':
                                        key = './/Date/'
                                        self.exact_date = True
                                    else:
                                        key = './/Days/'
                                    if i.find(key) != None:
                                        self.delivery_days = i.find(key).text

                            self.is_valid = True
                            self._calculated = True

                            #if needs_cache:
                            #    cache.set(cache_key_response, tree, 60)
        else:
            error = errors[0]
            err_num = error.find('.//Number').text
            source = error.find('.//Source').text
            description = error.find('.//Description').text
            log.info("USPS Error: Code %s: %s" % (err_num, description))

    def verbose_log(self, *args, **kwargs):
        if config_value('shipping.modules.usps', 'VERBOSE_LOG'):
            log.debug(*args, **kwargs)



from django_extensions.management.jobs import DailyJob
from product.models import ProductPriceLookup

class Job(DailyJob):
    help = "Update the pricing lookup table."

    def execute(self):
        ProductPriceLookup.objects.rebuild_all()


"""Base forms for use in newsletter subscribing"""

from django import forms
from django.utils.translation import ugettext_lazy as _
from satchmo_ext.newsletter import update_subscription
from satchmo_ext.newsletter.models import get_contact_or_fake

_NOTSET = object()

class OptionalBoolean(forms.BooleanField):
    def __init__(self, *args, **kwargs):
        forms.BooleanField.__init__(self, *args, **kwargs)
        self.required = False

class NewsletterForm(forms.Form):
    full_name = forms.CharField(label=_('Full Name'), max_length=100, required=False)
    email = forms.EmailField(label=_('Email address'), max_length=75, required=True)
    subscribed = OptionalBoolean(label=_('Subscribe'), required=False, initial=True)

    def get_contact(self):
        data = self.cleaned_data
        email = data['email']
        full_name = data['full_name']
        
        return get_contact_or_fake(full_name, email)
    
    def save(self, state=_NOTSET, attributes={}):
        contact = self.get_contact()
        if state == _NOTSET:
            state = self.cleaned_data['subscribed']
        return update_subscription(contact, state, attributes=attributes)



from django.utils.translation import ugettext_lazy as _
from satchmo_store import VERSION
from satchmo_store.contact.models import Contact
from satchmo_store.shop.models import Order, Cart
from satchmo_store.shop.signals import satchmo_context
from threaded_multihost import threadlocals
import datetime
import logging
import operator
from product.models import Product, ProductVariation


log = logging.getLogger('satchmo_toolbar')

def add_toolbar_context(sender, context={}, **kwargs):
    user = threadlocals.get_current_user()
    if user and user.is_staff:
        request_path = context['request'].META['PATH_INFO']
        slug = request_path.split('/')[-2]
        total_sales = 0
        show_sales = False
        variation_items = []
        try:
            product = Product.objects.get(slug=slug)
            show_sales = True
            subtypes = product.get_subtypes()
            if 'ConfigurableProduct' in subtypes:
                total_sales = 0
                all_variations = ProductVariation.objects.filter(parent=product)
                for variation in all_variations:
                    total_sales += variation.product.total_sold
                    variation_items.append(variation.product)
            else:
                total_sales = product.total_sold
                
        except:
            pass
            
        st = {}
        st['st_satchmo_version'] = VERSION
        newq = Order.objects.filter(status__exact = 'New')
        st['st_new_order_ct'] = newq.count()
        amounts = newq.values_list('total', flat=True)
        if amounts:
            newtotal = reduce(operator.add, amounts)
        else:
            newtotal = 0
        st['st_new_order_total'] = newtotal
        st['st_total_sold'] = total_sales
        st['st_show_sales'] = show_sales
        st['st_variations'] = variation_items
        week = datetime.datetime.today()-datetime.timedelta(days=7)
        day = datetime.datetime.today()-datetime.timedelta(days=1)
        hours = datetime.datetime.today()-datetime.timedelta(hours=1)
        cartweekq = Cart.objects.filter(date_time_created__gte=week)
        cartdayq = Cart.objects.filter(date_time_created__gte=day)
        carthourq = Cart.objects.filter(date_time_created__gte=hours)
        st['st_cart_7d_ct'] = cartweekq.count()
        st['st_cart_1d_ct'] = cartdayq.count()
        st['st_cart_1h_ct'] = carthourq.count()
        
        st['st_contacts_ct'] = Contact.objects.all().count()
        st['st_contacts_7d_ct'] = Contact.objects.filter(create_date__gte=week).count()
        # edits = []
        # st['st_edits'] = edits        
        
        context.update(st)
        

def start_listening():
    log.debug('Satchmo toolbar ready')
    satchmo_context.connect(add_toolbar_context)



from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.admin.views.decorators import staff_member_required
from django.views.decorators.cache import never_cache
from livesettings import ConfigurationSettings, forms
from livesettings.overrides import get_overrides
import logging

log = logging.getLogger('configuration.views')

def group_settings(request, group, template='livesettings/group_settings.html'):
    # Determine what set of settings this editor is used for
    
    use_db, overrides = get_overrides();
    
    mgr = ConfigurationSettings()
    if group is None:
        settings = mgr
        title = 'Site settings'
    else:
        settings = mgr[group]
        title = settings.name
        log.debug('title: %s', title)

    if use_db:
        # Create an editor customized for the current user
        #editor = forms.customized_editor(settings)

        if request.method == 'POST':
            # Populate the form with user-submitted data
            data = request.POST.copy()
            form = forms.SettingsEditor(data, settings=settings)
            if form.is_valid():
                form.full_clean()
                for name, value in form.cleaned_data.items():
                    group, key = name.split('__')
                    cfg = mgr.get_config(group, key)
                    if cfg.update(value):

                        # Give user feedback as to which settings were changed
                        request.user.message_set.create(message='Updated %s on %s' % (cfg.key, cfg.group.key))

                return HttpResponseRedirect(request.path)
        else:
            # Leave the form populated with current setting values
            #form = editor()
            form = forms.SettingsEditor(settings=settings)
    else:
        form = None
        
    return render_to_response(template, {
        'title': title,
        'group' : group,
        'form': form,
        'use_db' : use_db
    }, context_instance=RequestContext(request))
group_settings = never_cache(staff_member_required(group_settings))

# Site-wide setting editor is identical, but without a group
# staff_member_required is implied, since it calls group_settings
def site_settings(request):
    return group_settings(request, group=None, template='livesettings/site_settings.html')
    
def export_as_python(request):
    """Export site settings as a dictionary of dictionaries"""
    
    from livesettings.models import Setting, LongSetting
    import pprint
    
    work = {}
    both = list(Setting.objects.all())
    both.extend(list(LongSetting.objects.all()))
    
    for s in both:
        if not work.has_key(s.site.id):
            work[s.site.id] = {}
        sitesettings = work[s.site.id]
                    
        if not sitesettings.has_key(s.group):
            sitesettings[s.group] = {}
        sitegroup = sitesettings[s.group]
        
        sitegroup[s.key] = s.value
        
    pp = pprint.PrettyPrinter(indent=4)
    pretty = pp.pformat(work)

    return render_to_response('livesettings/text.txt', { 'text' : pretty }, mimetype='text/plain')
    
export_as_python = never_cache(staff_member_required(export_as_python))



"""
Urls for Product Brand module, note that you do not have to add this to your urls file, it will get automatically included by collect_urls.
"""
from django.conf.urls.defaults import *
from satchmo_store.shop import get_satchmo_setting
import logging

log = logging.getLogger('brand.urls')

urlpatterns = patterns('satchmo_ext.brand.views',
    (r'^$', 'brand_list', {}, 'satchmo_brand_list'),
    (r'^(?P<brandname>.*)/(?P<catname>.*)/$', 'brand_category_page', {}, 'satchmo_brand_category_view'),
    (r'^(?P<brandname>.*)/$', 'brand_page', {}, 'satchmo_brand_view'),
)

brandbase = r'^' + get_satchmo_setting('BRAND_SLUG') + '/'

prodbase = r'^' + get_satchmo_setting('PRODUCT_SLUG') + '/'
brandpatterns = patterns('',
    (brandbase, include('satchmo_ext.brand.urls'))
)

def add_brand_urls(sender, patterns=(), section="", **kwargs):
    if section=="__init__":
        log.debug('adding brand urls at %s', brandbase)
        patterns += brandpatterns



from django.utils.translation import ugettext_lazy as _
from livesettings import *

SHIP_MODULES = config_get('SHIPPING', 'MODULES')
SHIP_MODULES.add_choice(('shipping.modules.ups', 'UPS'))

SHIPPING_GROUP = ConfigurationGroup('shipping.modules.ups',
    _('UPS Shipping Settings'),
    requires = SHIP_MODULES,
    ordering = 101)

config_register_list(

    StringValue(SHIPPING_GROUP,
        'XML_KEY',
        description=_("UPS XML Access Key"),
        help_text=_("XML Access Key Provided by UPS"),
        default=u""),
    
    StringValue(SHIPPING_GROUP,
        'USER_ID',
        description=_("UPS User ID"),
        help_text=_("User ID provided by UPS site."),
        default=u""),

    StringValue(SHIPPING_GROUP,
        'ACCOUNT',
        description=_("UPS Account Number"),
        help_text=_("UPS Account Number."),
        default=u""),
    
    StringValue(SHIPPING_GROUP,
        'USER_PASSWORD',
        description=_("UPS User Password"),
        help_text=_("User password provided by UPS site."),
        default=u""),
    
    MultipleStringValue(SHIPPING_GROUP,
        'UPS_SHIPPING_CHOICES',
        description=_("UPS Shipping Choices Available to customers."),
        choices = (
            (('01', 'Next Day Air')),
            (('02', 'Second Day Air')),
            (('03', 'Ground')),
            (('12', '3 Day Select')),
            (('13', 'Next Day Air Saver')),
            (('11', 'Standard'))
        ),
        default = ('11',)),

    StringValue(SHIPPING_GROUP,
        'SHIPPING_CONTAINER',
        description=_("Type of container used to ship product."),
        choices = (
            (('00', 'Unknown')),
            (('01', 'UPS LETTER')),
            (('02', 'PACKAGE / CUSTOMER SUPPLIED')),
        ),
        default = u"00"),

    StringValue(SHIPPING_GROUP,
        'PICKUP_TYPE',
        description=_("UPS Pickup option."),
        choices = (
            (('01', 'DAILY PICKUP')),
            (('03', 'CUSTOMER COUNTER')),
            (('06', 'ONE TIME PICKUP')),
            (('07', 'ON CALL PICKUP')),
            ),
        default = u"07"),

    BooleanValue(SHIPPING_GROUP,
        'LIVE',
        description=_("Access production UPS server"),
        help_text=_("Use this when your store is in production."),
        default=False),

    StringValue(SHIPPING_GROUP,
        'CONNECTION',
        description=_("Submit to URL"),
        help_text=_("Address to submit live transactions."),
        default="https://onlinetools.ups.com/ups.app/xml/Rate"),

    StringValue(SHIPPING_GROUP,
        'CONNECTION_TEST',
        description=_("Submit to TestURL"),
        help_text=_("Address to submit test transactions."),
        default="https://wwwcie.ups.com/ups.app/xml/Rate"),

    BooleanValue(SHIPPING_GROUP,
        'VERBOSE_LOG',
        description=_("Verbose logs"),
        help_text=_("Send the entire request and response to the log - for debugging help when setting up UPS."),
        default=False)

)



'''
A versioning plugin for Elixir.

Entities that are marked as versioned with the `acts_as_versioned` statement
will automatically have a history table created and a timestamp and version
column added to their tables. In addition, versioned entities are provided
with four new methods: revert, revert_to, compare_with and get_as_of, and one
new attribute: versions.  Entities with compound primary keys are supported.

The `versions` attribute will contain a list of previous versions of the
instance, in increasing version number order.

The `get_as_of` method will retrieve a previous version of the instance "as of"
a specified datetime. If the current version is the most recent, it will be
returned.

The `revert` method will rollback the current instance to its previous version,
if possible. Once reverted, the current instance will be expired from the
session, and you will need to fetch it again to retrieve the now reverted
instance.

The `revert_to` method will rollback the current instance to the specified
version number, if possibe. Once reverted, the current instance will be expired
from the session, and you will need to fetch it again to retrieve the now
reverted instance.

The `compare_with` method will compare the instance with a previous version. A
dictionary will be returned with each field difference as an element in the
dictionary where the key is the field name and the value is a tuple of the
format (current_value, version_value). Version instances also have a
`compare_with` method so that two versions can be compared.

Also included in the module is a `after_revert` decorator that can be used to
decorate methods on the versioned entity that will be called following that
instance being reverted.

The acts_as_versioned statement also accepts an optional `ignore` argument
that consists of a list of strings, specifying names of fields.  Changes in
those fields will not result in a version increment.  In addition, you can
pass in an optional `check_concurrent` argument, which will use SQLAlchemy's
built-in optimistic concurrency mechanisms.

Note that relationships that are stored in mapping tables will not be included
as part of the versioning process, and will need to be handled manually. Only
values within the entity's main table will be versioned into the history table.
'''

from datetime              import datetime
import inspect

from sqlalchemy            import Table, Column, and_, desc
from sqlalchemy.orm        import mapper, MapperExtension, EXT_CONTINUE, \
                                  object_session

from elixir                import Integer, DateTime
from elixir.statements     import Statement
from elixir.properties     import EntityBuilder
from elixir.entity         import getmembers

__all__ = ['acts_as_versioned', 'after_revert']
__doc_all__ = []

#
# utility functions
#

def get_entity_where(instance):
    clauses = []
    for column in instance.table.primary_key.columns:
        instance_value = getattr(instance, column.name)
        clauses.append(column==instance_value)
    return and_(*clauses)


def get_history_where(instance):
    clauses = []
    history_columns = instance.__history_table__.primary_key.columns
    for column in instance.table.primary_key.columns:
        instance_value = getattr(instance, column.name)
        history_column = getattr(history_columns, column.name)
        clauses.append(history_column==instance_value)
    return and_(*clauses)


#
# a mapper extension to track versions on insert, update, and delete
#

class VersionedMapperExtension(MapperExtension):
    def before_insert(self, mapper, connection, instance):
        version_colname, timestamp_colname = \
            instance.__class__.__versioned_column_names__
        setattr(instance, version_colname, 1)
        setattr(instance, timestamp_colname, datetime.now())
        return EXT_CONTINUE

    def before_update(self, mapper, connection, instance):
        old_values = instance.table.select(get_entity_where(instance)) \
                                   .execute().fetchone()

        # SA might've flagged this for an update even though it didn't change.
        # This occurs when a relation is updated, thus marking this instance
        # for a save/update operation. We check here against the last version
        # to ensure we really should save this version and update the version
        # data.
        ignored = instance.__class__.__ignored_fields__
        version_colname, timestamp_colname = \
            instance.__class__.__versioned_column_names__
        for key in instance.table.c.keys():
            if key in ignored:
                continue
            if getattr(instance, key) != old_values[key]:
                # the instance was really updated, so we create a new version
                dict_values = dict(old_values.items())
                connection.execute(
                    instance.__class__.__history_table__.insert(), dict_values)
                old_version = getattr(instance, version_colname)
                setattr(instance, version_colname, old_version + 1)
                setattr(instance, timestamp_colname, datetime.now())
                break

        return EXT_CONTINUE

    def before_delete(self, mapper, connection, instance):
        connection.execute(instance.__history_table__.delete(
            get_history_where(instance)
        ))
        return EXT_CONTINUE


versioned_mapper_extension = VersionedMapperExtension()


#
# the acts_as_versioned statement
#

class VersionedEntityBuilder(EntityBuilder):

    def __init__(self, entity, ignore=None, check_concurrent=False,
                 column_names=None):
        self.entity = entity
        self.add_mapper_extension(versioned_mapper_extension)
        #TODO: we should rather check that the version_id_col isn't set
        # externally
        self.check_concurrent = check_concurrent

        # Changes in these fields will be ignored
        if column_names is None:
            column_names = ['version', 'timestamp']
        entity.__versioned_column_names__ = column_names
        if ignore is None:
            ignore = []
        ignore.extend(column_names)
        entity.__ignored_fields__ = ignore

    def create_non_pk_cols(self):
        # add a version column to the entity, along with a timestamp
        version_colname, timestamp_colname = \
            self.entity.__versioned_column_names__
        #XXX: fail in case the columns already exist?
        #col_names = [col.name for col in self.entity._descriptor.columns]
        #if version_colname not in col_names:
        self.add_table_column(Column(version_colname, Integer))
        #if timestamp_colname not in col_names:
        self.add_table_column(Column(timestamp_colname, DateTime))

        # add a concurrent_version column to the entity, if required
        if self.check_concurrent:
            self.entity._descriptor.version_id_col = 'concurrent_version'

    # we copy columns from the main entity table, so we need it to exist first
    def after_table(self):
        entity = self.entity
        version_colname, timestamp_colname = \
            entity.__versioned_column_names__

        # look for events
        after_revert_events = []
        for name, func in getmembers(entity, inspect.ismethod):
            if getattr(func, '_elixir_after_revert', False):
                after_revert_events.append(func)

        # create a history table for the entity
        skipped_columns = [version_colname]
        if self.check_concurrent:
            skipped_columns.append('concurrent_version')

        columns = [
            column.copy() for column in entity.table.c
            if column.name not in skipped_columns
        ]
        columns.append(Column(version_colname, Integer, primary_key=True))
        table = Table(entity.table.name + '_history', entity.table.metadata,
            *columns
        )
        entity.__history_table__ = table

        # create an object that represents a version of this entity
        class Version(object):
            pass

        # map the version class to the history table for this entity
        Version.__name__ = entity.__name__ + 'Version'
        Version.__versioned_entity__ = entity
        mapper(Version, entity.__history_table__)

        version_col = getattr(table.c, version_colname)
        timestamp_col = getattr(table.c, timestamp_colname)

        # attach utility methods and properties to the entity
        def get_versions(self):
            v = object_session(self).query(Version) \
                                    .filter(get_history_where(self)) \
                                    .order_by(version_col) \
                                    .all()
            # history contains all the previous records.
            # Add the current one to the list to get all the versions
            v.append(self)
            return v

        def get_as_of(self, dt):
            # if the passed in timestamp is older than our current version's
            # time stamp, then the most recent version is our current version
            if getattr(self, timestamp_colname) < dt:
                return self

            # otherwise, we need to look to the history table to get our
            # older version
            sess = object_session(self)
            query = sess.query(Version) \
                        .filter(and_(get_history_where(self),
                                     timestamp_col <= dt)) \
                        .order_by(desc(timestamp_col)).limit(1)
            return query.first()

        def revert_to(self, to_version):
            if isinstance(to_version, Version):
                to_version = getattr(to_version, version_colname)

            old_version = table.select(and_(
                get_history_where(self),
                version_col == to_version
            )).execute().fetchone()

            entity.table.update(get_entity_where(self)).execute(
                dict(old_version.items())
            )

            table.delete(and_(get_history_where(self),
                              version_col >= to_version)).execute()
            self.expire()
            for event in after_revert_events:
                event(self)

        def revert(self):
            assert getattr(self, version_colname) > 1
            self.revert_to(getattr(self, version_colname) - 1)

        def compare_with(self, version):
            differences = {}
            for column in self.table.c:
                if column.name in (version_colname, 'concurrent_version'):
                    continue
                this = getattr(self, column.name)
                that = getattr(version, column.name)
                if this != that:
                    differences[column.name] = (this, that)
            return differences

        entity.versions = property(get_versions)
        entity.get_as_of = get_as_of
        entity.revert_to = revert_to
        entity.revert = revert
        entity.compare_with = compare_with
        Version.compare_with = compare_with

acts_as_versioned = Statement(VersionedEntityBuilder)


def after_revert(func):
    """
    Decorator for watching for revert events.
    """
    func._elixir_after_revert = True
    return func





from livesettings import *
from django.utils.translation import ugettext_lazy as _

# this is so that the translation utility will pick up the string
gettext = lambda s: s
_strings = (gettext('CreditCard'), gettext('Credit Card'), gettext('Prot/X Secure Payments'))

# These cards require the issue number and start date fields filled in.
REQUIRES_ISSUE_NUMBER = ('MAESTRO', 'SOLO')

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_PROTX', 
    _('Prot/X Payment Settings'), 
    ordering=101)

config_register_list(

    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),
        
    BooleanValue(PAYMENT_GROUP, 
        'SIMULATOR', 
        description=_("Simulated Transactions?"),
        help_text=_("Must be false to accept real payments"),
        default=False),

    BooleanValue(PAYMENT_GROUP, 
        'SKIP_POST', 
        description=_("Skip post?"),
        help_text=_("For testing only, this will skip actually posting to Prot/x servers.  This is because their servers restrict IPs of posting servers, even for tests.  If you are developing on a desktop, you'll have to enable this."),
        default=False),
        
    StringValue(PAYMENT_GROUP, 
        'CAPTURE',
        description=_('Payment Capture'),
        help_text=_('This can be "Payment" which captures immediately, or "Deferred".  Note that you can only use the latter if you set option on your Prot/X account first.'),
        choices = (
            (('PAYMENT', 'Payment')),
            (('DEFERRED', 'Deferred')),
        ),
        default = 'PAYMENT'),
    
    
    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the checkout pages?"), 
        default=False),

    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.protx'),
    
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'PROTX'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Prot/X Secure Payments',
        help_text = _('This will be passed to the translation utility')),

    MultipleStringValue(PAYMENT_GROUP,
        'CREDITCHOICES',
        description=_('Available credit cards'),
        choices = (
                (('VISA','Visa Credit/Debit')),
                (('UKE','Visa Electron')),
                (('DELTA','Delta')),
                #(('AMEX','American Express')),  # not always available
                #(('DC','Diners Club')), # not always available
                (('MC','Mastercard')),
                (('MAESTRO','UK Maestro')),
                (('SOLO','Solo')),
                (('JCB','JCB')),
            ),
        default = ('VISA', 'MC')),
    
    StringValue(PAYMENT_GROUP, 
        'VENDOR', 
        description=_('Your Vendor Name'),
        default="",
        help_text= _("This is used for Live and Test transactions.  Make sure to add your server IP address to VSP, or it won't work.")),

    StringValue(PAYMENT_GROUP, 
        'VENDOR_SIMULATOR', 
        description=_('Simulator Vendor Name'),
        default="",
        help_text= _("This is used for Live and Test transactions.  Make sure to activate the VSP Simulator (you have to directly request it) and add your server IP address to the VSP Simulator, or it won't work.")),
            
    StringValue(PAYMENT_GROUP, 
        'CURRENCY_CODE',
        description=_('Currency Code'),
        help_text=_('Currency code for Prot/X transactions.'),
        default = 'GBP'),
        
    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = r'^protx/'),
        
    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)



"""Satchmo product rating views"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.contrib.comments.models import Comment
from django.contrib.comments.views.comments import post_comment
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponseRedirect, Http404
from django.contrib.sites.models import Site

from logging import getLogger

log = getLogger('satchmo_store.shop.views.ratings')

def post_rating(request, url='/ratings/posted/', maxcomments=1):
    """Wrap django.contrib.comments.views.comments.post_comment, so that we can control where the user
    is returned after submit, also add the ability to control the maximum number of ratings per user 
    per product.
    """
    if request.method != "POST":
        raise Http404, _("One or more of the required fields wasn't submitted")
        
    if request.POST.has_key('url'):
        url = request.POST['url']
        
    response = post_comment(request)

    if maxcomments > 0 and not request.user.is_anonymous():
        try:
            target = request.POST['target']
        except KeyError:
            raise Http404, _("One or more of the required fields wasn't submitted")
            
        content_type_id, object_id = target.split(':')
        
        try:
            ct = ContentType.objects.get(pk=content_type_id)
        except ContentType.DoesNotExist:
            raise Http404, _("Bad ContentType: %s" % content_type_id)
        
        comments = Comment.objects.filter(object_id__exact=object_id,
            content_type__app_label__exact=ct.app_label,
            content_type__model__exact=ct.model,
            site__exact=Site.objects.get_current(),
            is_public__exact=True,
            user__exact=request.user.id).order_by('submit_date')
            
        ct = len(comments)
        if ct > maxcomments:
            log.debug("Got %i comments for user - removing all but %i", ct, maxcomments)
            for c in comments:
                c.delete()
                ct -= 1
                if ct == maxcomments: break
    
    return HttpResponseRedirect(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from notmm.dbapi.orm import decorators
from notmm.http.decorators import post_required
from notmm.utils.template import direct_to_template

@post_required
@decorators.with_schevo_database('moviereviews')
def test_schevo_database(request, **kwds):
    db = request.environ['schevo.db.moviereviews']
    assert request.method is 'POST', request.method
    assert 'schevo.db.moviereviews' in request.environ.keys()
    assert db.format == 2, 'invalid db format: %d' % db.format
    assert hasattr(db, 'Q'), 'db has no Q attribute!'
    
    #Q = db.Q
    #movies = (item for item in db.Movie.find())
    db.close()
    return direct_to_template(request, template_name='sandbox/test.mako', 
        status=200)
    



from django.template import Library
from l10n.utils import moneyfmt

register = Library()

def option_price(option_item):
    """
    Returns the price as (+$1.00)
    or (-$1.00) depending on the sign of the price change
    The currency symbol is set in the settings.py file
    """
    output = ""
    if option_item.price_change != 0:
        amount = moneyfmt(abs(option_item.price_change))
    if option_item.price_change < 0:
        output = "(- %s)" % amount
    if option_item.price_change > 0:
        output = "(+ %s)" % amount
    return output

register.simple_tag(option_price)

def option_total_price(product, option_item):
    """
    Returns the price as (+$1.00)
    or (-$1.00) depending on the sign of the price change
    The currency symbol is set in the settings.py file
    """
    if option_item.price_change:
        val = product.unit_price + option_item.price_change
    else:
        val = product.unit_price
    return moneyfmt(val)

register.simple_tag(option_total_price)



#!/usr/bin/env python
"""manage.py script to perform management tasks"""

import sys,os
import django


# Hack for adding lib/sandbox to the pythonpath
currentdir = os.getcwd()

sys.path.append(os.path.join(currentdir, 'lib')) # test apps
sys.path.append(os.path.join(currentdir, 'lib/site-packages')) # contrib apps


def setup_environ():
    """Environment sanity checks"""
    required_keys = ('DJANGO_HOME', 'DJANGO_SETTINGS_MODULE')
    for name in required_keys:
        if not os.environ.has_key(name):
            # Raise an error since this is a requirement
            raise EnvironmentError('Please set %r' % name)
        else:
            print 'Found %s .. ok' % repr(name)
    # Add `DJANGO_HOME` to sys.path
    sys.path.append(os.environ['DJANGO_HOME'])

def init_main(argv=None, fallback='settings'):
    """A wrapper for execute_manager"""
    try:
        setup_environ()
        
        #module_name  = os.environ['DJANGO_SETTINGS_MODULE'] or fallback
        #settings_module = __import__(module_name, {}, {}, [])

    except (ImportError, EnvironmentError), e:
        raise e
    else:
        from notmm.dbapi.orm import management
        management.execute_from_command_line(argv)

if __name__ == "__main__":
    #django.setup()
    init_main(argv=sys.argv)




#!/usr/bin/env python
# -*- coding: utf-8 -*-


from notmm.utils.YAMLDct import RandomYAMLDct
#from mainapp.feeds.utils import get_feed

#def settings(request):
#    ctx = dict(settings=_settings)
#    return ctx

def request(request):
    ctx = dict(request=request)
    return ctx

def link_set(request):
    # custom links to put on every pages
    default_set =  RandomYAMLDct('lib/mainapp/static/content/1/free-software.yaml')

    dct = {
        'free_software' : default_set
        #'blogs-fr': YAMLDct('blogs-fr.yaml'),
        #'opendata': YAMLDct('opendata.yaml'),
        #'notmm-api': YAMLDct('notmm-api.yaml', randomized=False),
        #'webmaster-tools': YAMLDct('webmaster-tools.yaml'),
        #'truth-alliance': YAMLDct('truth-alliance.yaml')
        #'for-sale': YAMLDct('for-sale.yaml') ## livestore links!
    }
    return {'link_set' : dct}

#def rss(request):
#    dct = {
#        'rss' : get_feed('django-hotsauce')
#    }
#    return dct




"""Satchmo product signals

Signals:

 - `discount_validate`: Usage::
 
      discount_validate.send(sender=Discount, discount=self, cart=cart, 
      contact=contact, shipping_choices=shipping_choices, shipping=shipping, success=success)
      
      Listeners should modify the "success" dictionary to veto the discount validity.
 - `discount_filter_items`: Usage::

      discount_filter_items.send(sender=self, discounted=discounted, order=order)

      Listeners should modify the "discounted" dictionary to change the set of discounted cart
      items.
      
 - `index_prerender`: Usage::

      index_prerender.send(Sender, request=request, context=ctx, object_list=somelist)
   
 - `satchmo_price_query`: Usage::
 
      satchmo_price_query.send(self, adjustment=PriceAdjustmentCalc) 

 - `satchmo_order_success`
"""

import django.dispatch

discount_validate = django.dispatch.Signal()
discount_filter_items = django.dispatch.Signal()
index_prerender = django.dispatch.Signal()
satchmo_price_query = django.dispatch.Signal()
subtype_order_success = django.dispatch.Signal()



"""
This dummy module can be used as a basis for creating your own

- Copy this module to a new name
- Make the changes described below
"""

# Note, make sure you use decimal math everywhere!
from decimal import Decimal
from django.utils.translation import ugettext as _
from shipping.modules.base import BaseShipper

class Shipper(BaseShipper):

    flatRateFee = Decimal("15.00")
    id = "Dumy"
        
    def __str__(self):
        """
        This is mainly helpful for debugging purposes
        """
        return "Dummy Flat Rate"
        
    def description(self):
        """
        A basic description that will be displayed to the user when selecting their shipping options
        """
        return _("Dummy Flat Rate Shipping")

    def cost(self):
        """
        Complex calculations can be done here as long as the return value is a decimal figure
        """
        assert(self._calculated)
        return(self.flatRateFee)

    def method(self):
        """
        Describes the actual delivery service (Mail, FedEx, DHL, UPS, etc)
        """
        return _("US Mail")

    def expectedDelivery(self):
        """
        Can be a plain string or complex calcuation returning an actual date
        """
        return _("3 - 4 business days")

    def valid(self, order=None):
        """
        Can do complex validation about whether or not this option is valid.
        For example, may check to see if the recipient is in an allowed country
        or location.
        """
        return True




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging

from schevo.backends.zodb import ZodbBackend # require zodb 3.7.4
from ._databaseproxy import DatabaseProxy

log = logging.getLogger(__name__)

__all__ = ('ClientStorageProxy',)

class ClientStorageProxy(DatabaseProxy):
    def __init__(self, db_name, **kwargs):
        # XXX Hack.
        if not 'db_connection_cls' in kwargs:
            kwargs['db_connection_cls'] = ZodbBackend
        super(ClientStorageProxy, self).__init__(db_name, **kwargs)




from django.db import models
try:
    from django.utils import simplejson
except ImportError:
    import simplejson
from django.utils.translation import ugettext_lazy as _
from listeners import wishlist_cart_add_listener
from satchmo_store import shop
from satchmo_store.contact.models import Contact
from product.models import Product
from satchmo_store.shop.signals import cart_add_view
from signals_ahoy.signals import collect_urls
import datetime

class ProductWishManager(models.Manager):
    def create_if_new(self, product, contact, details):
        encoded = simplejson.dumps(details)

        products = ProductWish.objects.filter(product=product, contact=contact, _details=encoded)
        if products and len(products) > 0:
            wish = products[0]
            if len(products) > 1:
                for p in products[1:]:
                    p.delete()
        else:
            wish = ProductWish(product=product, contact=contact)
            wish.details = details
            wish.save()
            
        return wish        

class ProductWish(models.Model):
    contact = models.ForeignKey(Contact, verbose_name=_("Contact"), related_name="contacts")
    product = models.ForeignKey(Product, verbose_name=_("Product"), related_name="products")
    _details = models.TextField(_('Details'), null=True, blank=True)
    create_date = models.DateTimeField(_("Creation Date"))
    
    objects = ProductWishManager()

    def set_details(self, raw):
        """Set the details from a raw list"""
        if raw:
            self._details = simplejson.dumps(raw)
    
    def get_details(self):
        """Convert the pickled details into a list"""
        if self._details:
            return simplejson.loads(self._details)
        else:
            return []

    details = property(fget=get_details, fset=set_details)

    def save(self, **kwargs):
        """Ensure we have a create_date before saving the first time."""
        if not self.pk:
            self.create_date = datetime.date.today()
        super(ProductWish, self).save(**kwargs)
        
    class Meta:
        verbose_name = _('Product Wish')
        verbose_name_plural = _('Product Wishes')

cart_add_view.connect(wishlist_cart_add_listener)

import config
from urls import add_wishlist_urls
collect_urls.connect(add_wishlist_urls, sender=shop)



from django.conf import settings as djangosettings
from django.utils.translation import ugettext
from livesettings import values
from livesettings.models import SettingNotSet
from satchmo_utils import flatten_list, is_list_or_tuple, is_string_like, load_module

import logging

log = logging.getLogger('configuration')

_NOTSET = object()

class ConfigurationSettings(object):
    """A singleton manager for ConfigurationSettings"""

    class __impl(object):
        def __init__(self):
            self.settings = values.SortedDotDict()
            self.prereg = {}

        def __getitem__(self, key):
            """Get an element either by ConfigurationGroup object or by its key"""
            key = self._resolve_key(key)
            return self.settings.get(key)

        def __getattr__(self, key):
            """Get an element either by ConfigurationGroup object or by its key"""
            try:
                return self[key]
            except:
                raise AttributeError, key
            
        def __iter__(self):
            for v in self.groups():
                yield v
            
        def __len__(self):
            return len(self.settings)

        def __contains__(self, key):
            try:
                key = self._resolve_key(key)
                return self.settings.has_key(key)
            except:
                return False

        def _resolve_key(self, raw):
            if is_string_like(raw):
                key = raw

            elif isinstance(raw, values.ConfigurationGroup):
                key = raw.key
                
            else:
                group = self.groups()[raw]
                key = group.key

            return key
                        
        def get_config(self, group, key):
            try:
                if isinstance(group, values.ConfigurationGroup):
                    group = group.key

                cg = self.settings.get(group, None)
                if not cg:
                    raise SettingNotSet('%s config group does not exist' % group)
                
                else:
                    return cg[key]
            except KeyError:
                raise SettingNotSet('%s.%s' % (group, key))

        def groups(self):
            """Return ordered list"""
            return self.settings.values()
                        
        def has_config(self, group, key):
            if isinstance(group, values.ConfigurationGroup):
                group = group.key
            
            cfg = self.settings.get(group, None)
            if cfg and key in cfg:
                return True
            else:
                return False
                        
        def preregister_choice(self, group, key, choice):
            """Setup a choice for a group/key which hasn't been instantiated yet."""
            k = (group, key)
            if self.prereg.has_key(k):
                self.prereg[k].append(choice)
            else:
                self.prereg[k] = [choice]
            
        def register(self, value):
            g = value.group
            if not isinstance(g, values.ConfigurationGroup):
                raise ValueError('value.group should be an instance of ConfigurationGroup')
                
            groupkey = g.key
            valuekey = value.key
            
            k = (groupkey, valuekey)
            if self.prereg.has_key(k):
                for choice in self.prereg[k]:
                    value.add_choice(choice)
            
            if not groupkey in self.settings:
                self.settings[groupkey] = g
                
            self.settings[groupkey][valuekey] = value                
            
            return value
            
    __instance = None

    def __init__(self):
        if ConfigurationSettings.__instance is None:
            ConfigurationSettings.__instance = ConfigurationSettings.__impl()
            #ConfigurationSettings.__instance.load_app_configurations()
        
        self.__dict__['_ConfigurationSettings__instance'] = ConfigurationSettings.__instance

    def __getattr__(self, attr):
            """ Delegate access to implementation """
            return getattr(self.__instance, attr)

    def __getitem__(self, key):
        return self.__instance[key]

    def __len__(self):
        return len(self.__instance)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)

    def __unicode__(self):
        return u"ConfigurationSettings: " + unicode(self.groups())

def config_exists(group, key):
    """Test to see if a setting has been registered"""

    return ConfigurationSettings().has_config(group, key)
    
def config_get(group, key):
    """Get a configuration setting"""
    try:
        return ConfigurationSettings().get_config(group, key)
    except SettingNotSet:
        log.debug('SettingNotSet: %s.%s', group, key)
        raise
    
def config_get_group(group):
    return ConfigurationSettings()[group]

def config_collect_values(group, groupkey, key, unique=True, skip_missing=True):
    """Look up (group, groupkey) from config, then take the values returned and
    use them as groups for a second-stage lookup.
    
    For example:
    
    config_collect_values(PAYMENT, MODULES, CREDITCHOICES)
    
    Stage 1: ['PAYMENT_GOOGLE', 'PAYMENT_AUTHORIZENET']
    Stage 2: config_value('PAYMENT_GOOGLE', 'CREDITCHOICES')
           + config_value('PAYMENT_AUTHORIZENET', 'CREDITCHOICES')
    Stage 3: (if unique is true) remove dupes
    """
    groups = config_value(group, groupkey)
    
    ret = []
    for g in groups:
        try:
            ret.append(config_value(g, key))
        except KeyError, ke:
            if not skip_missing:
                raise SettingNotSet('No config %s.%s' % (g, key))
    
    if unique:
        out = []
        for x in ret:
            if not x in out:
                out.append(x)
        ret = out
    
    return ret
    
def config_register(value):
    """Register a value or values.

    Parameters:
        -A Value
    """
    return ConfigurationSettings().register(value)
    
def config_register_list(*args):
    for value in args:
        config_register(value)
    
def config_value(group, key, default=_NOTSET):
    """Get a value from the configuration system"""
    try:
        return config_get(group, key).value
    except SettingNotSet:
        if default != _NOTSET:
            return default
        raise
    
def config_value_safe(group, key, default_value):
    """Get a config value with a default fallback, safe for use during SyncDB."""
    raw = default_value

    try:
        raw = config_value(group, key)
    except SettingNotSet:
        pass
    except ImportError, e:
        log.warn("Error getting %s.%s, OK if you are in SyncDB.", group, key)

    return raw

    
def config_choice_values(group, key, skip_missing=True, translate=False):
    """Get pairs of key, label from the setting."""
    try:
        cfg = config_get(group, key)
        choices = cfg.choice_values
        
    except SettingNotSet:
        if skip_missing:
            return []
        else:
            raise SettingNotSet('%s.%s' % (group, key))
            
    if translate:
        choices = [(k, ugettext(v)) for k, v in choices]
        
    return choices

def config_add_choice(group, key, choice):
    """Add a choice to a value"""
    if config_exists(group, key):
        cfg = config_get(group, key)
        cfg.add_choice(choice)
    else:
        ConfigurationSettings().preregister_choice(group, key, choice)



#!/usr/bin/env python
import unittest, operator
from utils import get_feed,get_feed_choices

class tm_feeds_TestCase(unittest.TestCase):
    
    def test_get_feed(self):
        feed = get_feed()

    def test_with_integers(self):
        feed_1 = get_feed(feed_id=1)
        feed_2 = get_feed(feed_id=2)
        self.failIfEqual(feed_1, feed_2)
    
    def test_get_feed_choices(self):
        data = get_feed_choices()
        self.assertEqual(isinstance(data, dict), True, type(data))

        #feed_choices = get_feed_choices()
        #aList = list(map(operator.itemgetter(2,1), feed_choices))
        #self.assertNotEqual(aList, feed_choices)

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2008 Etienne Robillard  <robillard.etienne@gmail.com>
# mainapp.py

#try:
#    import psyco
#    psyco.full()
#except ImportError:
#    pass

import os
import sys
#import webob
#from authkit.authenticate import middleware as authentication_middleware

#from tm.configuration import settings

from notmm.utils.wsgi           import IterableWSGIResponse
from notmm.utils.cutandpaste    import make_server, get_bind_addr
from notmm.utils.cutandpaste    import Banner
from notmm.utils.configparse    import loadconf
from notmm.controllers import *
#from wsgiref.validator import validate

from django.http import HttpRequest, HttpResponse, CompatCookie
from django.core.handlers.wsgi import WSGIRequest, WSGIHandler
from django.contrib.auth.models import AnonymousUser
from django.template import TemplateDoesNotExist, TemplateSyntaxError


class SatchmoController(WSGIController):

    def process_request(self, env):
        self._environ.update(env)
        self._request = self.request_class(env)
        self._request.environ = env.copy()
        #self._request.session = self.session

        self._request.path = env['PATH_INFO']
 
        from django.utils.importlib import import_module
        engine = import_module(self.settings.SESSION_ENGINE)
        session_key = self._request.COOKIES.get(self.settings.SESSION_COOKIE_NAME, None)
        self._request.session = engine.SessionStore(session_key)
        self._request.user = AnonymousUser()
        self._request.cookies = CompatCookie()
        
    def environ_getter(self):
        
        return self._environ

    environ = property(environ_getter)
    
    def __call__(self, env, start_response):

        try:
            self.process_request(env)
            response = super(SatchmoController, self).get_response(env, start_response)
        except (TemplateDoesNotExist, TemplateSyntaxError):
            #response = self.handle500(self.request)        
            #return response
            return process_exception(self.request, self.response_class)(env, start_response)
        start_response('200 OK', [('Content-Type', 'text/html')])
        return response



def exit(c):
    return os._exit(c)

def main(argv):
    """ a wrapper for running runserver """
    
    app_conf = loadconf('development.ini')
    
    # authkit user configuration variables are stored here
    auth_conf = app_conf['auth'] 

    wsgi_app = SatchmoController(
        global_conf='local_settings',
        request_class=WSGIRequest,
        response_class=HttpResponse,
        environ=app_conf, 
        #maintainer_mode=True
        )
    
    # Setup default callback functions to invoke for trapping
    # common exceptions
    #wsgi_app.sethandle('handle400', 'tm.handlers.handle400')
    #wsgi_app.sethandle('handle401', 'tm.handlers.handle401') # authkit needs this
    wsgi_app.sethandle('handle404', 'django.views.defaults.page_not_found')
    wsgi_app.sethandle('handle500', 'django.views.defaults.server_error')
    
    # Install the authentication middleware (LoginController)
    # wsgi_app = LoginController(wsgi_app, auth_conf)
    wsgi_app = SessionController(wsgi_app)

    bind_addr = get_bind_addr(app_conf)
    
    # Startup banner
    banner = Banner(sys.stderr, bind_addr=bind_addr)
    banner.show()
    
    conn = make_server(wsgi_app, bind_addr)
    try:
        conn.serve_forever()
    except (KeyboardInterrupt, SystemExit):
        if 'close' in conn.__dict__:
            conn.close()
        print
        print 'bye.'
        exit(2)
    except IOError:
        raise

    exit(0)
        

if __name__ == "__main__":
   #assert len(sys.argv) >= 2, 'usage: runserver.py host:port' 
   main(argv=sys.argv)




__all__ = [
    'before_insert',
    'after_insert',
    'before_update',
    'after_update',
    'before_delete',
    'after_delete',
    'reconstructor'
]

def create_decorator(event_name):
    def decorator(func):
        if not hasattr(func, '_elixir_events'):
            func._elixir_events = []
        func._elixir_events.append(event_name)
        return func
    return decorator

before_insert = create_decorator('before_insert')
after_insert = create_decorator('after_insert')
before_update = create_decorator('before_update')
after_update = create_decorator('after_update')
before_delete = create_decorator('before_delete')
after_delete = create_decorator('after_delete')
try:
    from sqlalchemy.orm import reconstructor
except ImportError:
    def reconstructor(func):
        raise Exception('The reconstructor method decorator is only '
                        'available with SQLAlchemy 0.5 and later')



from django.conf import settings
from django.template import loader, Context, TemplateDoesNotExist
from livesettings import config_value
from satchmo_store.shop.signals import rendering_store_mail, sending_store_mail

from socket import error as SocketError

import logging
log = logging.getLogger('satchmo_store.mail')

if "mailer" in settings.INSTALLED_APPS:
    from mailer import send_mail
else:
    from django.core.mail import send_mail

from django.core.mail import EmailMultiAlternatives

class NoRecipientsException(StandardError):
    pass

class ShouldNotSendMail(Exception):
    pass

def send_store_mail_template_decorator(template_base):
    """
    This decorator sets the arguments ``template`` and ``template_html``
    when the decorated function is called.
    """
    def dec(func):
        def newfunc(*args, **kwargs):
            default_kwargs = {
                'template': '%s.txt' % template_base,
                'template_html': '%s.html' % template_base
            }
            default_kwargs.update(kwargs)
            return func(*args, **default_kwargs)
        return newfunc
    return dec

def send_html_email(sender, send_mail_args=None, context=None, template_html=None ,**kwargs):
    send_html = config_value('SHOP', 'HTML_EMAIL')

    if not send_html:
        return

    # perhaps send_store_mail() was not passed the *template_html* argument
    if not template_html:
        return

    if settings.DEBUG:
        log.info("Attempting to send html mail.")
    try:
        t = loader.get_template(template_html)
        html_body = t.render(context)
    except TemplateDoesNotExist:
        log.warn('Unable to find html email template %s. Falling back to text only email.' % template_html)
        return

    # just like send_store_mail() does
    if not send_mail_args.get('recipient_list'):
        raise NoRecipientsException

    # prepare kwargs for EmailMultiAlternatives()
    fail_silently = send_mail_args.pop('fail_silently')
    send_mail_args['body'] = send_mail_args.pop('message') # the plain text part
    send_mail_args['to'] = send_mail_args.pop('recipient_list')

    msg = EmailMultiAlternatives(**send_mail_args)
    msg.attach_alternative(html_body, "text/html")

    # don't have to handle any errors, as send_store_mail() does so for us.
    msg.send(fail_silently=fail_silently)

    # tell send_store_mail() to abort sending plain text mail
    raise ShouldNotSendMail

def send_store_mail(subject, context, template='', recipients_list=None,
                    format_subject=False, send_to_store=False,
                    fail_silently=False, sender=None, **kwargs):
    """
    :param subject: A string.

    :param format_subject: Determines whether the *subject* parameter is
      formatted. Only the %(shop_name)s specifier is supported now.

    :param context: A dictionary to use when rendering the message body. This
      overwrites an internal dictionary with a single entry, `shop_name`.

    :param template: The path of the plain text template to use when rendering
      the message body.

    :param `**kwargs`: Additional arguments that are passed to listeners of the
      signal :data:`satchmo_store.shop.signals.sending_store_mail`.
    """
    from satchmo_store.shop.models import Config

    shop_config = Config.objects.get_current()
    shop_email = shop_config.store_email
    shop_name = shop_config.store_name
    if not shop_email:
        log.warn('No email address configured for the shop.  Using admin settings.')
        shop_email = settings.ADMINS[0][1]

    c_dict = {'shop_name': shop_name}

    if format_subject:
        subject = subject % c_dict

    c_dict.update(context)
    c = Context(c_dict)

    recipients = recipients_list or []

    if send_to_store:
        recipients.append(shop_email)

    # match send_mail's signature
    send_mail_args = {
        'subject': subject,
        'from_email': shop_email,
        'recipient_list': recipients,
        'fail_silently': fail_silently,
    }

    # let listeners modify context
    rendering_store_mail.send(sender, send_mail_args=send_mail_args, context=c,
                              **kwargs)

    # render text email, regardless of whether html email is used.
    t = loader.get_template(template)
    body = t.render(c)

    # listeners may have set this entry
    if not 'message' in send_mail_args:
        send_mail_args['message'] = body

    try:
        # We inform listeners before checking recipients list, as they may
        # modify it.
        # Listeners may also choose to send mail themselves, so we place this
        # call in the SocketError try block to handle errors for them.
        try:
            sending_store_mail.send(sender, send_mail_args=send_mail_args, \
                                    context=c, **kwargs)
        except ShouldNotSendMail:
            return

        if not recipients:
            raise NoRecipientsException

        send_mail(**send_mail_args)
    except SocketError, e:
        if settings.DEBUG:
            log.error('Error sending mail: %s' % e)
            log.warn("""Ignoring email error, since you are running in DEBUG mode.  Email was:
To: %s
Subject: %s
---
%s""" % (",".join(send_mail_args['recipient_list']), send_mail_args['subject'], send_mail_args['message']))
        else:
            log.fatal('Error sending mail: %s' % e)
            raise IOError('Could not send email. Please make sure your email settings are correct and that you are not being blocked by your ISP.')



from datetime import datetime, timedelta
from decimal import Decimal
from livesettings import config_get_group
from payment import active_gateways
from satchmo_store.shop.models import Order, OrderAuthorization, OrderItem, OrderItemDetail, OrderPayment, OrderPendingPayment
from satchmo_store.shop.signals import satchmo_post_copy_item_to_order
from shipping.utils import update_shipping
from socket import error as SocketError
import logging

log = logging.getLogger('payment.utils')

def capture_authorizations(order):
    """Capture all outstanding authorizations on this order"""
    if order.authorized_remaining > Decimal('0'):
        purchase = order.get_or_create_purchase()
        for key, group in active_gateways():
            gateway_settings = config_get(group, 'MODULE')
            processor = get_gateway_by_settings(gateway_settings)
            processor.capture_authorized_payments(purchase)

def get_or_create_order(request, working_cart, contact, data):
    """Get the existing order from the session, else create using 
    the working_cart, contact and data"""
    shipping = data.get('shipping', None)
    discount = data.get('discount', None)
    
    try:
        order = Order.objects.from_request(request)
        if order.status != '':
            # This order is being processed. We should not touch it!
            order = None
    except Order.DoesNotExist:
        order = None

    update = bool(order)
    if order:
        # make sure to copy/update addresses - they may have changed
        order.copy_addresses() 
        order.save()
        if discount is None and order.discount_code:
            discount = order.discount_code
    else:
        # Create a new order.
        order = Order(contact=contact)

    pay_ship_save(order, working_cart, contact,
        shipping=shipping, discount=discount, update=update)
    request.session['orderID'] = order.id
    return order

def get_gateway_by_settings(gateway_settings, settings={}):
    log.debug('getting gateway by settings: %s', gateway_settings.key)
    processor_module = gateway_settings.MODULE.load_module('processor')
    gateway_settings = get_gateway_settings(gateway_settings, settings=settings)
    return processor_module.PaymentProcessor(settings=gateway_settings)

def get_processor_by_key(key):
    payment_module = config_get_group(key)
    processor_module = payment_module.MODULE.load_module('processor')
    return processor_module.PaymentProcessor(payment_module)

def pay_ship_save(new_order, cart, contact, shipping, discount, update=False):
    """
    Save the order details, first removing all items if this is an update.
    """
    if shipping:
        update_shipping(new_order, shipping, contact, cart)

    if not update:
        # Temp setting of the tax and total so we can save it
        new_order.total = Decimal('0.00')
        new_order.tax = Decimal('0.00')
        new_order.sub_total = cart.total
        new_order.method = 'Online'

    if discount:
        new_order.discount_code = discount
    else:
        new_order.discount_code = ""

    update_orderitems(new_order, cart, update=update)

def update_orderitem_details(new_order_item, item):
    """Update orderitem details, if any.
    """
    if item.has_details:
        # Check to see if cartitem has CartItemDetails
        # If so, add here.
        #obj = CustomTextField.objects.get(id=item.details.values()[0]['customfield_id'])
        #val = item.details.values()[0]['detail']
        for detail in item.details.all():
            new_details = OrderItemDetail(item=new_order_item,
                value=detail.value,
                name=detail.name,
                price_change=detail.price_change,
                sort_order=detail.sort_order)
            new_details.save()


def update_orderitem_for_subscription(new_order_item, item):
    """Update orderitem subscription details, if any.
    """
    #if product is recurring, set subscription end
    #if item.product.expire_length:
    if item.product.is_subscription:
        subscription = item.product.subscriptionproduct
        if subscription.expire_length:
            new_order_item.expire_date = subscription.calc_expire_date()
    else:
        subscription = None

    #if product has trial price, set it here and update expire_date with trial period.
    trial = None

    if subscription:
        trial = subscription.get_trial_terms()

    if trial:
        trial1 = trial[0]
        new_order_item.unit_price = trial1.price
        new_order_item.line_item_price = new_order_item.quantity * new_order_item.unit_price
        new_order_item.expire_date = trial1.calc_expire_date()

    new_order_item.save()


def update_orderitems(new_order, cart, update=False):
    """Update the order with all cart items, first removing all items if this
    is an update.
    """
    if update:
        new_order.remove_all_items()
    else:
        # have to save first, or else we can't add orderitems
        new_order.site = cart.site
        new_order.save()

    # Add all the items in the cart to the order
    for item in cart.cartitem_set.all():
        new_order_item = OrderItem(order=new_order,
            product=item.product,
            quantity=item.quantity,
            unit_price=item.unit_price,
            line_item_price=item.line_total)

        update_orderitem_for_subscription(new_order_item, item)
        update_orderitem_details(new_order_item, item)

        # Send a signal after copying items
        # External applications can copy their related objects using this
        satchmo_post_copy_item_to_order.send(
                cart,
                cartitem=item,
                order=new_order, orderitem=new_order_item
                )

    new_order.recalculate_total()



from datetime import datetime
from decimal import Decimal
from django.test import TestCase
from models import Carrier, QuantityTier, Shipper

def make_tiers(carrier, prices, expires=None):
    for qty, handling, price in prices:
        t = QuantityTier(carrier=carrier, 
            handling=Decimal("%i.00" % handling),
            price=Decimal("%i.00" % price),
            quantity=Decimal(qty),
            expires=expires
        )
        t.save()
    

class TieredCarrierSimpleTest(TestCase):
    fixtures = []

    def testCreate(self):
        c = Carrier(key="test", active=True)
        c.save()
        t = QuantityTier(carrier=c, 
            quantity=Decimal('1'),
            handling=Decimal("10.00"),
            price=Decimal("0.00"),
            )
        t.save()
        
        self.assertEqual(c.price(1), Decimal("10.00"))
        self.assertEqual(c.price(4), Decimal("10.00"))
        
class TieredCarrierPricingTest(TestCase):
    fixtures = []

    def setUp(self):
        self.carrier = Carrier(name="pricing", active=True)
        self.carrier.save()
        t = QuantityTier(carrier=self.carrier, 
            quantity=Decimal('1'),
            handling=Decimal("10.00"),
            price=Decimal("0.00"),
            )
        t.save()
        
    def testBase(self):
        self.assertEqual(self.carrier.price(1), Decimal("10.00"))
        self.assertEqual(self.carrier.price(10), Decimal("10.00"))
        self.assertEqual(self.carrier.price(100), Decimal("10.00"))
        
    def test2Prices(self):
        t = QuantityTier(carrier=self.carrier, 
            quantity=Decimal('10'),
            handling=Decimal("100.00"),
            price=Decimal("1.00"),
            )
        t.save()
        
        self.assertEqual(self.carrier.price(1), Decimal("10.00"))
        self.assertEqual(self.carrier.price(9), Decimal("10.00"))
        self.assertEqual(self.carrier.price(10), Decimal("110.00"))
        self.assertEqual(self.carrier.price(100), Decimal("200.00"))

class TieredCarrierExpiringTest(TestCase):
    fixtures = []

    def setUp(self):
        self.carrier = Carrier(name="pricing", active=True)
        self.carrier.save()

        base_prices = (
            (1, 10, 0),
            (20, 20, 1),
            (30, 30, 2),
            (40, 40, 1)
        )
        make_tiers(self.carrier, base_prices)
        
    def testExpired(self):
        dt = datetime(2000, 1, 1)
        
        sale_prices = (
            (1, 1, 0),
            (20, 2, 1),
            (30, 3, 1),
            (40, 4, 1)
        )
        
        make_tiers(self.carrier, sale_prices, expires=dt)

        self.assertEqual(self.carrier.price(1), Decimal("10.00"))
        self.assertEqual(self.carrier.price(20), Decimal("40.00"))
        self.assertEqual(self.carrier.price(30), Decimal("90.00"))
        self.assertEqual(self.carrier.price(100), Decimal("140.00"))

    def testNotExpired(self):
        
        now = datetime.now()
        nextyear = datetime(now.year+1, now.month, now.day)
        sale_prices = (
            (1, 1, 0),
            (20, 2, 1),
            (30, 3, 1),
            (40, 4, 1)
        )
        
        make_tiers(self.carrier, sale_prices, expires=nextyear)

        self.assertEqual(self.carrier.price(1), Decimal("1.00"))
        self.assertEqual(self.carrier.price(10), Decimal("1.00"))
        self.assertEqual(self.carrier.price(20), Decimal("22.00"))
        self.assertEqual(self.carrier.price(30), Decimal("33.00"))
        self.assertEqual(self.carrier.price(100), Decimal("104.00"))



import posixpath
import io

from notmm.utils.wsgilib import HTTPResponse
from .configuration import settings

def image_handler(request, *args, **kwds):
    docroot = kwds.get('document_root', settings.MEDIA_ROOT)
    filename = posixpath.split(request.path_url)[-1]
    ext = filename.split('.')[-1]

    filename = posixpath.join(docroot, filename)

    fdata = io.open(filename, 'rb')
    outbuf = fdata.read()
    headers = (
      ('Content-Type', "image/%s" % ext),
      ('Content-Length', str(len(outbuf)))
    )
    return HTTPResponse(content=outbuf, headers=headers, force_unicode=False)




from notmm.utils.configparse     import string_getter
from notmm.utils.template        import *
from test_support import unittest, app_conf, settings

import posixpath

class TemplateLoaderFactoryTestCase(unittest.TestCase):
    
    def setUp(self):
        TemplateLoaderFactory.configure(
            kwargs={'directories': settings.TEMPLATE_DIRS, 
                    'cache_enabled': settings.ENABLE_BEAKER})
        self.loader = TemplateLoaderFactory.get_loader()
        

    def test_get_template_loader(self):
        # parent class must match the expected templates backend
        loader = self.loader
        
        #self.assertEqual(isinstance(self.loader, TemplateLoader), True)
    
        # See if this loader has defined minimal hooks
        self.assertEqual(hasattr(loader, 'get_template'), True)
        self.assertEqual(hasattr(loader, 'has_template'), True)

class TemplateLoaderTestCase(TemplateLoaderFactoryTestCase):

    def test_get_template(self):
        # Find a relative template by name
        t = 'sandbox/request_context.mako'
        template = self.loader.get_template(t)
        uri = template.uri
        self.assertEqual(uri, t)
        self.assertEqual(self.loader.has_template(uri), True)
        
        head, tail = posixpath.split(uri)
        self.assertEqual(head, 'sandbox')
        self.assertEqual(posixpath.basename(template.filename), tail)

        # available templates directories 
        self.assertEqual(len(self.loader.directories)>=1, True)
    



from decimal import Decimal, InvalidOperation
from django import template
from django.utils.safestring import mark_safe
from l10n.utils import moneyfmt
from satchmo_utils.templatetags import get_filter_args

import logging

log = logging.getLogger("satchmo_currency")

register = template.Library()

def currency(value, args=""):
    """Convert a value to a money formatted string.

    places:  required number of places after the decimal point
    curr:    optional currency symbol before the sign (may be blank)
    wrapcents:tag to wrap the part after the decimal point

    Usage:
        val|currency
        val|currency:'places=2'
        val|currency:'places=2:wrapcents=sup'
    """

    if value == '' or value is None:
        return value

    args, kwargs = get_filter_args(args,
        keywords=('places','curr', 'wrapcents'),
        intargs=('places',), stripquotes=True)

    try:
        value = Decimal(str(value))
    except InvalidOperation:
        log.error("Could not convert value '%s' to decimal", value)
        raise

    return mark_safe(moneyfmt(value, **kwargs))

register.filter('currency', currency)
currency.is_safe = True



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from kiwi.environ import environ

class Cursor(object):
    plus = gtk.gdk.pixbuf_new_from_file(
        environ.find_resource('pixmap', 'plus.png'))
    hotspot = (6, 6)
    offset = (8, 8)
    size = (32, 32)
    fallback = gtk.gdk.Cursor(gtk.gdk.PLUS)

    #@classmethod
    def create(cls, pixbuf):
        display = gtk.gdk.display_get_default()
        if not pixbuf or not display.supports_cursor_color():
            # On Windows, the patch from Bug #306101 must be applied to
            # enable cursor color support (and alpha support on XP).
            return cls.fallback

        tmp = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, *cls.size)
        tmp.fill(0)
        cls.plus.composite(tmp, 0, 0,
                           cls.plus.get_width(),
                           cls.plus.get_height(),
                           0, 0, 1, 1, gtk.gdk.INTERP_NEAREST, 255)
        x, y = cls.offset
        w, h = pixbuf.get_width(), pixbuf.get_height()
        if x + w >= cls.size[0]:
            x = cls.size[0] - w
        if y + h >= cls.size[1]:
            y = cls.size[1] - h

        pixbuf.composite(tmp,
                         x, y, w, h,
                         x, y, 1, 1,
                         gtk.gdk.INTERP_NEAREST, 255)

        return gtk.gdk.Cursor(display, tmp, *cls.hotspot)
    create = classmethod(create)

    #@classmethod
    def set_for_widget_adaptor(cls, window, widget_adaptor):
        if not widget_adaptor:
            cursor = None
        else:
            cursor = widget_adaptor.cursor
        window.set_cursor(cursor)
    set_for_widget_adaptor = classmethod(set_for_widget_adaptor)
